#ifndef CLASS_ELEMENT
#define CLASS_ELEMENT

#include <vector>

#include "graphic/SDLImage.h"
#include "IsoObject.h"
#include "StatisticalValue.h"

#define NUM_VIEWS 8

/* views */
#define NORTH 0
#define NORTH_EAST 1
#define EAST 2
#define SOUTH_EAST 3
#define SOUTH 4
#define SOUTH_WEST 5
#define WEST 6
#define NORTH_WEST 7

#define SELF -1
#define ANY 8

/* stats */
// energy
#define STAT_ENERGY 0
// visibility range
#define STAT_VISIBILITY 1
// action points
#define STAT_ACTION 2

/* types of element */
#define SCENE_ELEMENT 0
#define BUILDING 1
#define UNIT 2

/// A base class for elements that can be present on the landscape
/// DO NOT USE this class for create objects, USE its subclasses (Unit, SceneElement, Building) 
class Element : public IsoObject
{
	protected:
	 /// images of the element
	 std::vector < graphic::Image * > _views;
	 /// statiticals data
	 std::vector < StatisticalValue * > _stats;
	 /// type of the element
	 int _type;
	 /// direction of the element
	 int _direction;

	public:
	 /// base constructor
	 Element () { };
	 /// constructor
	 Element(int r, int c, std::vector < StatisticalValue * > stats);
	 /// full constructor
	 Element(int r, int c, std::vector < graphic::Image * > views, std::vector < StatisticalValue * > stats);
	 /// destroyer
	 virtual ~Element();

	 /// set the image that should be blitted
	 virtual void SetDirection(int direction)
	 {
		_direction = direction;
		_img = _views[direction];
	 };

	 /// return direction value
	 int GetDirection() { return _direction; };

	 /* STATS MANAGEMENT */
	 /// return the max of a statistical value
	 int GetMaxStat(int id) { return _stats[id]->GetMax(); }
	 /// set the max of a statistical value
	 void SetMaxStat(int id, int val) { _stats[id]->SetMax(val); };
	 /// add a value to the max statistical value
	 void AddMaxStat(int id, int val) { _stats[id]->AddMax(val); };
	 /// subtract a value from the max statistical value
	 void SubMaxStat(int id, int val) { _stats[id]->SubMax(val); };
	 /// return the current  value of a statistical value
	 int GetCurStat(int id) { return _stats[id]->GetCur(); }
	 /// set the current value of a statistical value
	 void SetCurStat(int id, int val) { _stats[id]->SetCur(val); };
	 /// add a value to the current statistical value
	 void AddCurStat(int id, int val) { _stats[id]->AddCur(val); };
	 /// subtract a value from the current statistical value
	 void SubCurStat(int id, int val) { _stats[id]->SubCur(val); };
	 /// restore the current value to the max
	 void RestoreCurStat(int id) { _stats[id]->RestoreCur(); };

	 /// return the type of the element
	 int GetType() { return _type;};
};

#endif
