#include "Functions.h"

using namespace graphic;

// return screen coordinates from iso ones origin represents the origin of
// the isometric plane
FloatPoint * iso2scr(Point iso, Point origin)
{
	return iso2scr(iso.x, iso.y, origin.x, origin.y);
}

// return screen coordinates from iso ones origin represents the origin of
// the isometric plane
FloatPoint * iso2scr(Sint16 ix, Sint16 iy, Sint16 ox, Sint16 oy)
{
	FloatPoint * scr = new FloatPoint;

	//initial translation
	ix -= ox;
	iy -= oy;

	//back to 45
	iy *= 2;

	//back to orthogonal
	scr->x = ix * COS_MIN_45 - iy * SIN_MIN_45;
	scr->y = ix * SIN_MIN_45 + iy * COS_MIN_45;

	return scr;
}

// return the relative orientation of an origin point respect another
int RelDirection(Point * origin, Point * direction)
{
	return RelDirection(origin->x, origin->y, direction->x, direction->y);
}

// return the relative orientation of an origin cell respect another
int RelDirection(Sint16 ox, Sint16 oy, Sint16 dx, Sint16 dy)
{
	FloatPoint dir;
	int d;

	dir.x = dx - ox;
	dir.y = dy - oy;

	if((dir.x >= (- dir.y / TAN_67_5)) && (dir.x <= (- dir.y / TAN_22_5)) && (dir.y <= 0))
		d = NORTH_EAST;
	else if((dir.x >= (dir.y / TAN_67_5)) && (dir.x <= (- dir.y / TAN_67_5)) && (dir.y <= 0))
		d = NORTH;
	else if((dir.y >= (- dir.x / TAN_67_5)) && (dir.y <= ( dir.x / TAN_67_5)) && (dir.x >= 0))
		d = EAST;
	else if((dir.y <= (dir.x * TAN_67_5)) && (dir.y >= (dir.x / TAN_67_5)) && (dir.x >= 0))
		d = SOUTH_EAST;
	else if((dir.x <= (- dir.y / TAN_67_5)) && (dir.x >= (- dir.y / TAN_22_5)) && (dir.y > 0))
		d= SOUTH_WEST;
	else if((dir.x <= (dir.y / TAN_67_5)) && (dir.x >= (- dir.y / TAN_67_5)) && (dir.y > 0))
		d = SOUTH;
	else if((dir.y <= (- dir.x / TAN_67_5)) && (dir.y >= ( dir.x / TAN_67_5)) && (dir.x < 0))
		d =  WEST;
	else
		d = NORTH_WEST;

	return d;
}

// temporary menu
void menu(Options & op)
{
	// used colors
	SDL_Color sel_color = {0xED, 0x70, 0x00, 0};
	SDL_Color unsel_color = {0xAA, 0x60, 0x00, 0};
	SDL_Color help_color = {0xAA, 0xAA, 0xAA, 0};

	// background images
	Image * bg_800, * bg_1024, * bg;
	// menu text
	Text * menu_txt_s, * menu_txt_u;
	// help text
	Text * help_txt1, * help_txt2;

	bool use_sdl = true;

	#ifdef WITH_OPENGL
	if(screen->GetGraphicMode() == GL_GRAPHIC)
		use_sdl = false;
	#endif

	try
	{
		if(use_sdl)
		{
			bg_800 = new SDLImage("data/img/mars1_800.png");
			bg_1024 = new SDLImage("data/img/mars1_1024.png");

			menu_txt_s = new SDLText("data/font/Distres2.ttf", sel_color , DIM_FONT_MENU);
			menu_txt_u = new SDLText("data/font/Distres2.ttf", unsel_color , DIM_FONT_MENU);

			help_txt1 = new SDLText("data/font/tahomabd.ttf", help_color , DIM_FONT_HELP1);
			help_txt2 = new SDLText("data/font/tahomabd.ttf", help_color , DIM_FONT_HELP2);
		}
		#ifdef WITH_OPENGL
		else
		{
			bg_800 = new GLImage("data/img/mars1_800.png");
			bg_1024 = new GLImage("data/img/mars1_1024.png");

			menu_txt_s = new GLText("data/font/Distres2.ttf", sel_color , DIM_FONT_MENU);
			menu_txt_u = new GLText("data/font/Distres2.ttf", unsel_color , DIM_FONT_MENU);

			help_txt1 = new GLText("data/font/tahomabd.ttf", help_color , DIM_FONT_HELP1);
			help_txt2 = new GLText("data/font/tahomabd.ttf", help_color , DIM_FONT_HELP2);
		}
		#endif
	}
	catch(Exception e)
	{
		e.PrintError();
		exit(-1);
	}

	bg = bg_800;

	bool done = false;
	bool need_update = true;
	SDL_Event event;

	op.choice = 1;

	//game loop
	while(!done)
	{
		while(SDL_PollEvent(&event))
		{
			// exit if someone press the x simbol of the window with the mouse
			if( event.type == SDL_QUIT )
			{
				op.choice = 0;
				done = true;
			}
			else if(event.type == SDL_KEYDOWN)
			{
				switch(event.key.keysym.sym)
				{
					// ESC ==> exit
					case SDLK_ESCAPE:
						op.choice = 0;

						done = true;
						break;

					// go in fullscreen mode
					case SDLK_f:
						screen->GoFullScreen();

						#ifdef WIN32
						bg->UpdateTex();
						menu_txt_s->UpdateTexs(false);
						menu_txt_u->UpdateTexs(false);
						help_txt1->UpdateTexs(false);
						help_txt2->UpdateTexs(false);
						#endif

						need_update = true;
						break;

					// go in window mode
					case SDLK_w:
						screen->GoWindowed();

						#ifdef WIN32
						bg->UpdateTex(false);
						menu_txt_s->UpdateTexs(false);
						menu_txt_u->UpdateTexs(false);
						help_txt1->UpdateTexs(false);
						help_txt2->UpdateTexs(false);
						#endif

						need_update = true;
						break;

					// end
					case SDLK_RETURN:
						done = true;
						break;

					// 800x600
					case SDLK_F1:
						screen->SetResolution(RIS1_W, RIS1_H);

						bg = bg_800;

						#ifdef WIN32
						bg->UpdateTex(false);
						menu_txt_s->UpdateTexs(false);
						menu_txt_u->UpdateTexs(false);
						help_txt1->UpdateTexs(false);
						help_txt2->UpdateTexs(false);
						#endif

						op.ris.w = RIS1_W;
						op.ris.h = RIS1_H;

						need_update = true;
						break;

					// 1024x768
					case SDLK_F2:
						screen->SetResolution(RIS2_W, RIS2_H);

						bg = bg_1024;

						#ifdef WIN32
						bg->UpdateTex(false);
						menu_txt_s->UpdateTexs(false);
						menu_txt_u->UpdateTexs(false);
						help_txt1->UpdateTexs(false);
						help_txt2->UpdateTexs(false);
						#endif

						op.ris.w = RIS2_W;
						op.ris.h = RIS2_H;

						need_update = true;
						break;

					case SDLK_UP:
						op.choice = op.choice > 1 ? (op.choice - 1) : 3;

						need_update = true;
						break;

					case SDLK_DOWN:
						op.choice =  op.choice < 3 ? (op.choice + 1)  : 1;

						need_update = true;
						break;

					default:
						break;
				}
			}
		}

		// update the scene
		if(need_update || !use_sdl)
		{
			screen->Clear();

			blitter->Blit(bg);

			if(op.choice == 1)
				menu_txt_s->WriteText("Tileset 1", MENU_X_OFFSET, op.ris.h / 3);
			else
				menu_txt_u->WriteText("Tileset 1", MENU_X_OFFSET, op.ris.h / 3);
			if(op.choice == 2)
				menu_txt_s->WriteText("Tileset 2", MENU_X_OFFSET, op.ris.h / 3 + 40);
			else
				menu_txt_u->WriteText("Tileset 2", MENU_X_OFFSET, op.ris.h / 3 + 40);
			if(op.choice == 3)
				menu_txt_s->WriteText("Tileset 3", MENU_X_OFFSET, op.ris.h / 3 + 80);
			else
				menu_txt_u->WriteText("Tileset 3", MENU_X_OFFSET, op.ris.h / 3 + 80);

			help_txt1->WriteText("Commands:", HELP_X_OFFSET, op.ris.h - 160);
			help_txt2->WriteText("F1 / F2 => 800x600 / 1024x768\nw / f => window / fullscreen\n\
							1, 2 => change mouse pointer (test)\ng => grid ON/OFF\n\
							arrows => scrolling\n\
							SPACE => restore unit AP\nmouse left click => select/move unit\n\
							right click => rotate unit", HELP_X_OFFSET2, op.ris.h - 140);

			screen->Update();

			need_update = false;
		}

		SDL_Delay(20);
	}

	// free used resources
	delete bg_800;
	delete bg_1024;
	delete menu_txt_s;
	delete menu_txt_u;
	delete help_txt1;
	delete help_txt2; 
}
