#include "SelectionData.h"

using namespace graphic;

// constructor
SelectionData::SelectionData(std::vector < Image * > images)
{
	int num_imgs = images.size();

	// store images pointers
	for(int i = 0; i < num_imgs; i++)
		_images.push_back(images[i]);

	// no obj selected by default
	_sel_elem = NULL;
}

// blit part of the layer on a surface, if the surface is NULL blit on screen
void SelectionData::BlitLayer(SDL_Rect limits_rect, Surface * dst_surf)
{
	if(_sel_elem == NULL)
		return ;

	Sint16 l_limit = limits_rect.x ;
	Sint16 u_limit = limits_rect.y;

	Image * obj_img = _sel_elem->GetImage();

	// temporary Point
	Point p;

	p.x = _sel_elem->GetX() +
			 ((obj_img->GetVisibleX() + obj_img->GetVisibleW() - _images[BAR_BORDER_IMG]->GetW())/2) ;
	p.y = _sel_elem->GetY() + obj_img->GetVisibleY() - BORDER_V_OFFSET;

	// check if the bar is inside the scene
	if(p.x + _images[BAR_BORDER_IMG]->GetW() >= l_limit && p.y + _images[BAR_BORDER_IMG]->GetH()>=  u_limit)
	{
		_images[BAR_BORDER_IMG]->SetPosition(p.x - l_limit, p.y - u_limit);

		//blit border
		blitter->Blit(_images[BAR_BORDER_IMG], dst_surf);

		// blit all bad bar under the good one
		_images[BAR_BAD_IMG]->SetPosition(p.x + BAR_H_OFFSET - l_limit, p.y + BAR_V_OFFSET - u_limit);

		blitter->Blit(_images[BAR_BAD_IMG], dst_surf);

		// calculate energy scale
		int scale = _sel_elem->GetMaxStat(STAT_ENERGY) / _images[BAR_GOOD_IMG]->GetW();

		if(!scale)
			scale = 1;

		SDL_Rect fill_r, dest_r;

		// blit part of the good bar according to element energy
		fill_r.x = 0;
		fill_r.y = 0;
		fill_r.w = _sel_elem->GetCurStat(STAT_ENERGY) / scale;
		fill_r.h = _images[BAR_GOOD_IMG]->GetH();

		dest_r.x = p.x + BAR_H_OFFSET - l_limit;
		dest_r.y = p.y + BAR_V_OFFSET - u_limit;

		if(fill_r.w > 0)
			blitter->BlitPart(_images[BAR_GOOD_IMG], fill_r, dest_r, dst_surf);
	}
}

// update all the loaded images
void SelectionData::UpdateTexs(bool del_old_tex)
{
	int num_img = _images.size();

	for(int i = 0; i < num_img; i++)
		_images[i]->UpdateTex(del_old_tex);
}
