#ifndef CLASS_BLITTER
#define CLASS_BLITTER

#include "Surface.h"

namespace graphic
{

/// general blitting interface
///
/// with the current code you CAN'T blit a RGBA surface on a RGB surface with colorkey
/// this problem won't be fixed until we need it working, this for 2 reasons:
/// 1- I'm a lazy coder and I hate to do useless things
/// 2- fix it requires work (see point 1) and can make worse performance
///
/// m3xican
class Blitter
{
	public:
	 /// base contructor
	 Blitter() { };
	 /// destroyer
	 virtual ~Blitter() { };

	 // NULL -> NULL
	 /// copy an entire surface on another one of the same size
	 virtual void Copy(Surface * src, Surface * dest = NULL) = 0;

	 // NULL -> Rect
	 /// blit an entire surface on another one using surface position
	 virtual void Blit(Surface * src, Surface * dest = NULL) = 0;

	 // NULL -> Rect
	 /// blit an entire surface on another one NOT using surface position, but dest_rect one
	 virtual void Blit(Surface * src, SDL_Rect & dest_rect, Surface * dest = NULL) = 0;

	 // Rect -> Rect
	 /// blit part of a surface on another one  using surface position for the destination
	 virtual void BlitPart(Surface * src, SDL_Rect & src_rect, Surface * dest = NULL) = 0;

	 // Rect -> Rect
	 /// blit part of a surface on part of another one NOT using surface position
	 /// for the destination, but dest_rect.x and dest_rect.y
	 virtual void BlitPart(Surface * src, SDL_Rect & src_rect,
					   SDL_Rect & dest_rect, Surface * dest = NULL) = 0;

	 /// blit the part of screen limited by src_rect on the dest surface
	 virtual void SaveScreen(SDL_Rect & src_rect, Surface * dest) = 0;
	 /// blit part of screen limited by src_rect on part of the dest surface
	 /// limited by dest_rect
	 virtual void SaveScreen(SDL_Rect & src_rect, SDL_Rect & dest_rect,
							 Surface * dest) = 0;
};

/// global pointer
extern Blitter * blitter;

}

#endif
