#include "GLBlitter.h"

// Blit an entire surface on screen using surface position (NULL, -> Rect)
void graphic::GLBlitter::Blit(Surface * src, Surface * dest)
{
	GLSurface * gl_src = dynamic_cast<GLSurface *> (src);

	if(dest == NULL)
	{
		// update the src texture if it needs an update
		if(gl_src->NeedTexUpdate())
			gl_src->UpdateTex();

		glBindTexture(GL_TEXTURE_2D, gl_src->GetTexNum());
		glMatrixMode(GL_TEXTURE);
		glLoadIdentity();
		glOrtho(0, 2*gl_src->GetWReal(), 0, 2*gl_src->GetHReal(), -1, 1);
		glMatrixMode(GL_MODELVIEW);
		glColor4f(1.0f, 1.0f, 1.0f, gl_src->GetOpacity()/255.0f);

		int xpad = (gl_src->GetWReal() - gl_src->GetW())/2;
		int ypad = (gl_src->GetHReal() - gl_src->GetH())/2;

		glPushMatrix();
		glLoadIdentity();

		// Move to the upper left coerner of the image
		glTranslated(gl_src->GetX(), gl_src->GetY(), 0);

		// Blitting
		glBegin(GL_QUADS);
			glTexCoord2d(xpad, ypad);
				glVertex2d(0, 0);
			glTexCoord2d(xpad + gl_src->GetW(), ypad);
				glVertex2d(gl_src->GetW(), 0);
			glTexCoord2d(xpad + gl_src->GetW(), ypad + gl_src->GetH());
				glVertex2d(gl_src->GetW(), gl_src->GetH());
			glTexCoord2d(xpad, ypad + gl_src->GetH());
				glVertex2d(0, gl_src->GetH());
		glEnd();

		glPopMatrix();
	}

	else
	{
		GLSurface * gl_dest = dynamic_cast<GLSurface * >(dest);

		// backup rect
		SDL_Rect to;
		SDL_Rect * src_r = gl_src->_rectangle;

		// copy src surface rect values into backup rect
		to.x = src_r->x;
		to.y = src_r->y;
		to.w = src_r->w;
		to.h = src_r->h;

		// blit on dest
		SDL_BlitSurface(gl_src->_surface, NULL, gl_dest->_surface, &to);

		// request texture update
		gl_dest->RequestTexUpdate();
	}
}

// NULL -> Rect
// blit an entire surface on another one NOT using surface position, but dest_rect one
void graphic::GLBlitter::Blit(Surface * src, SDL_Rect & dest_rect, Surface * dest)
{
	GLSurface * gl_src = dynamic_cast<GLSurface *> (src);

	if(dest == NULL)
	{
		// update the src texture if it needs an update
		if(gl_src->NeedTexUpdate())
			gl_src->UpdateTex();

		glBindTexture(GL_TEXTURE_2D, gl_src->GetTexNum());
		glMatrixMode(GL_TEXTURE);
		glLoadIdentity();
		glOrtho(0, 2*gl_src->GetWReal(), 0, 2*gl_src->GetHReal(), -1, 1);
		glMatrixMode(GL_MODELVIEW);
		glColor4f(1.0f, 1.0f, 1.0f, gl_src->GetOpacity()/255.0f);

		int xpad = (gl_src->GetWReal() - gl_src->GetW())/2;
		int ypad = (gl_src->GetHReal() - gl_src->GetH())/2;

		glPushMatrix();
		glLoadIdentity();

		// Move to the upper left corner of the destination rectangle
		glTranslated(dest_rect.x, dest_rect.y, 0);

		// Blitting
		glBegin(GL_QUADS);
			glTexCoord2d(xpad, ypad);
				glVertex2d(0, 0);
			glTexCoord2d(xpad + gl_src->GetW(), ypad);
				glVertex2d(gl_src->GetW(), 0);
			glTexCoord2d(xpad + gl_src->GetW(), ypad + gl_src->GetH());
				glVertex2d(gl_src->GetW(), gl_src->GetH());
			glTexCoord2d(xpad, ypad + gl_src->GetH());
				glVertex2d(0, gl_src->GetH());
		glEnd();

		glPopMatrix();
	}

	else
	{
		GLSurface * gl_dest = dynamic_cast<GLSurface * >(dest);

		// backup rect
		SDL_Rect to;

		// copy src surface rect values into backup rect
		to.x = dest_rect.x;
		to.y = dest_rect.y;
		to.w = dest_rect.w;
		to.h = dest_rect.h;

		// blit on dest
		SDL_BlitSurface(gl_src->_surface, NULL, gl_dest->_surface, &to);

		// request texture update
		gl_dest->RequestTexUpdate();
	}
}

// Blit part of a surface on screen using surface position for the
// destination (Rect -> Rect)
void graphic::GLBlitter::BlitPart(Surface * src, SDL_Rect & src_rect, Surface * dest)
{
	GLSurface * gl_src = dynamic_cast<GLSurface *> (src);

	if(dest == NULL)
	{
		// update the src texture if it needs an update
		if(gl_src->NeedTexUpdate())
			gl_src->UpdateTex();

		glBindTexture(GL_TEXTURE_2D, gl_src->GetTexNum());
		glMatrixMode(GL_TEXTURE);
		glLoadIdentity();
		glOrtho(0, 2*gl_src->GetWReal(), 0, 2*gl_src->GetHReal(), -1, 1);
		glMatrixMode(GL_MODELVIEW);
		glColor4f(1.0f, 1.0f, 1.0f, gl_src->GetOpacity()/255.0f);

		int xpad = (gl_src->GetWReal() - gl_src->GetW())/2;
		int ypad = (gl_src->GetHReal() - gl_src->GetH())/2;
		
		glPushMatrix();
		glLoadIdentity();

		// Move to the upper left corner of the image part
		glTranslated(gl_src->GetX(), gl_src->GetY(), 0);

		// Blitting
		glBegin(GL_QUADS);
			glTexCoord2d(xpad + src_rect.x, ypad + src_rect.y);
				glVertex2d(0, 0);
			glTexCoord2d(xpad + src_rect.x + src_rect.w, ypad + src_rect.y);
				glVertex2d(src_rect.w, 0);
			glTexCoord2d(xpad + src_rect.x + src_rect.w, ypad + src_rect.y + src_rect.h);
				glVertex2d(src_rect.w, src_rect.h);
			glTexCoord2d(xpad + src_rect.x, ypad + src_rect.y + src_rect.h);
				glVertex2d(0, src_rect.h);
		glEnd();

		glPopMatrix();
	}

	else
	{
		GLSurface * gl_dest = dynamic_cast<GLSurface * >(dest);

		// backup rect
		SDL_Rect from, to;
		SDL_Rect * src_r = gl_src->_rectangle;

		// copy src surface rect values into backup rect
		from.x = src_rect.x;
	    from.y = src_rect.y;
    	from.w = src_rect.w;
	    from.h = src_rect.h;

		to.x = src_r->x;
		to.y = src_r->y;
		to.w = src_r->w;
		to.h = src_r->h;

		// blit on dest
		SDL_BlitSurface(gl_src->_surface, &from, gl_dest->_surface, &to);

		// request texture update
		gl_dest->RequestTexUpdate();
	}
}

// Blit part of a surface on part of screen NOT using surface position
// for the destination, but dest_rect.x and dest_rect.y (Rect -> Rect)
void graphic::GLBlitter::BlitPart(Surface * src, SDL_Rect & src_rect, SDL_Rect & dest_rect, Surface * dest)
{
	GLSurface * gl_src = dynamic_cast<GLSurface *> (src);

	if(dest == NULL)
	{
		// update the src texture if it needs an update
		if(gl_src->NeedTexUpdate())
			gl_src->UpdateTex();

		glBindTexture(GL_TEXTURE_2D, gl_src->GetTexNum());
		glMatrixMode(GL_TEXTURE);
		glLoadIdentity();
		glOrtho(0, 2*gl_src->GetWReal(), 0, 2*gl_src->GetHReal(), -1, 1);
		glMatrixMode(GL_MODELVIEW);
		glColor4f(1.0f, 1.0f, 1.0f, gl_src->GetOpacity()/255.0f);

		int xpad = (gl_src->GetWReal() - gl_src->GetW())/2;
		int ypad = (gl_src->GetHReal() - gl_src->GetH())/2;

		glPushMatrix();
		glLoadIdentity();

		// Move to the upper left corner of the destination rectangle
		glTranslated(dest_rect.x, dest_rect.y, 0);

		// Blitting
		glBegin(GL_QUADS);
			glTexCoord2d(xpad + src_rect.x, ypad + src_rect.y);
				glVertex2d(0, 0);
			glTexCoord2d(xpad + src_rect.x + src_rect.w, ypad + src_rect.y);
				glVertex2d(src_rect.w, 0);
			glTexCoord2d(xpad + src_rect.x + src_rect.w, ypad + src_rect.y + src_rect.h);
				glVertex2d(src_rect.w, src_rect.h);
			glTexCoord2d(xpad + src_rect.x, ypad + src_rect.y + src_rect.h);
				glVertex2d(0, src_rect.h);
		glEnd();

		glPopMatrix();
	}
	
	else
	{
		GLSurface * gl_dest = dynamic_cast<GLSurface * >(dest);

		// backup rect
		SDL_Rect from, to;

		// copy src surface rect values into backup rect
		from.x = src_rect.x;
	    from.y = src_rect.y;
    	from.w = src_rect.w;
	    from.h = src_rect.h;

		to.x = dest_rect.x;
		to.y = dest_rect.y;
		to.w = dest_rect.w;
		to.h = dest_rect.h;

		// blit on dest
		SDL_BlitSurface(gl_src->_surface, &from, gl_dest->_surface, &to);

		// request texture update
		gl_dest->RequestTexUpdate();
	}
}
