#include "GLScreen.h"

// init screen surface
graphic::GLScreen::GLScreen(Uint16 w, Uint16 h, Uint32 flags)
{
	// saving flags value adding the OpenGL SDL flag (in case it misses)
	_flags = flags | SDL_OPENGL;

	// setting screen dimensions
	_width = w;
	_height = h;

	InitScreen();
}

// init the whole GL graphic contest
void graphic::GLScreen::InitScreen()
{
	// setting OpenGL attributes
	SDL_GL_SetAttribute(SDL_GL_DOUBLEBUFFER, 1);

	// setting screen mode, get a screen from SDL
	_surface = SDL_SetVideoMode(_width, _height, 0, _flags);

	// setting projection matrix
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glOrtho(0, _width, _height, 0, -1, 1);
	glMatrixMode(GL_MODELVIEW);
	glViewport(0, 0, _width, _height);

	// setting clear color
	glClearColor(0.0f, 0.0f, 0.0f, 0.0f);

	// enabling texturing as everything is textured
	glEnable(GL_TEXTURE_2D);

	// enabling blending for transparency effects
	glEnable(GL_BLEND);

	// disabling depth test
	glDisable(GL_DEPTH);

	// setting blending function
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

	// an error occurred ==> throw an Exception object
	if(_surface == NULL)
		throw Exception("GLScreen", SDL_GetError());
}

// fill the surface with a color
void graphic::GLScreen::Fill(SDL_Color & color)
{
	// Fill
	glPushAttrib( GL_COLOR_BUFFER_BIT );
	glClearColor(color.r/255.0f, color.g/255.0f, color.b/255.0f, 0.0f);
	glClear(GL_COLOR_BUFFER_BIT);
	glPopAttrib();
}

// set screen resolution
void graphic::GLScreen::SetResolution(Uint16 w, Uint16 h)
{
	// change resolution just if it really changes
	if(w != _width || h != _height)
	{
		// Setting screen dimensions
		_width = w;
		_height = h;

		InitScreen();
	}
}

// set window view mode
void graphic::GLScreen::GoWindowed()
{
	// set window mode just if the screen is fullscreen
	if(_flags & SDL_FULLSCREEN)
	{
		_flags &= ~SDL_FULLSCREEN;

		InitScreen();
	}
}

// set fullscreen view mode
void graphic::GLScreen::GoFullScreen()
{
	// set fullscreen just if the screen is in window mode
	if(!(_flags & SDL_FULLSCREEN))
	{
		_flags |= SDL_FULLSCREEN;

		InitScreen();
	}
}

// set screen flags
void graphic::GLScreen::SetFlags(Uint32 flags)
{
	// saving flags value adding the OpenGL SDL flag (in case it misses)
	_flags = flags | SDL_OPENGL;

	InitScreen();
}
