#include "GLText.h"

// base constructor, init all the characters surfaces
graphic::GLText::GLText(std::string font, SDL_Color color, int size)
{
	TTF_Font * ttf_font;
	SDL_Surface * sdl_surf;
	GLSurface * surf;
	char c[] = " ";

	// Loading the font
	ttf_font = TTF_OpenFont(font.c_str(), size);
	// An error occurred loading the font
	if(ttf_font == NULL)
		throw Exception("GLText", TTF_GetError());

	for(int i = 0; i < NUM_CHARS ; i++)
	{
		// Rendering the font on the SDL_Surface
		sdl_surf = TTF_RenderText_Blended(ttf_font, c, color);
		// An error occurred rendering the character surface
		if(sdl_surf == NULL)
			throw Exception("GLText", TTF_GetError());

		surf = new GLSurface(sdl_surf);
		_surfaces[i] = surf;

		c[0]++;
	}

	// Close the font
	TTF_CloseFont(ttf_font);
}

// write a text starting from (x, y) on a Surface
//void graphic::GLText::WriteText(const std::string & text, Sint16 x, Sint16 y, Surface * dest)
void graphic::GLText::WriteText(std::string text, Sint16 x, Sint16 y, Surface * dest)
{
	Sint16 text_x = x;
	Sint16 text_y = y;

	int len = text.length();
	int ind;

	for(int i = 0; i < len; i++)
	{
		if(text[i] == '\n')
		{
			text_x = x;
			text_y += _surfaces[0]->GetH();
		}
		else
		{
			ind = (text[i] & 0xff) - ' ';

			if(ind >=  0 && ind <= NUM_CHARS)
			{
				_surfaces[ind]->SetPosition(text_x, text_y);

				blitter->Blit(_surfaces[ind], dest);

				text_x += _surfaces[ind]->GetW();
			}
		}
	}
}

// return the width of a text
Uint16 graphic::GLText::GetTextW(std::string txt)
{
	Uint16 max_len = 0;
	Uint16 len = 0;

	int str_len = txt.length();
	int char_ind;

	// Read all the chars
	for(int i = 0; i < str_len; i++)
	{
		// End of the line, check for its length
		if(txt[i] == '\n')
		{
			if(len > max_len)
				max_len = len;

			len = 0;
		}
		else
		{
			char_ind = (txt[i] & 0xff) - ' ';

			// Add the width of this char
			if(char_ind >=  0 && char_ind <= NUM_CHARS)
				len +=_surfaces[char_ind]->GetW();
		}
	}

	if(len > max_len)
		max_len = len;

	return max_len;
}

