#ifndef CLASS_GL_TEXT
#define CLASS_GL_TEXT

#include "GLSurface.h"
#include "GLBlitter.h"
#include "Text.h"

namespace graphic
{

/// A surface containing a text created with SDL_ttf
class GLText : public Text
{
	private:
	 /// GLsurfaces of all the characters
	 GLSurface * _surfaces[NUM_CHARS];

	public:
	 /// base constructor, init all the characters surfaces
	 GLText(std::string font, SDL_Color color, int size);
	 /// destructor
	 ~GLText()
	 {
		 for(int i = 0; i < NUM_CHARS; i++)
			 delete _surfaces[i];
	 };

	 /// write a text starting from (x, y) on a Surface
	 void WriteText(std::string text, Sint16 x, Sint16 y, Surface * dest = NULL);

	 /// Set the opacity of all the characters
	 void SetOpacity(Uint8 opacity)
	 {
		 _opacity = opacity;

		 for(int i = 0; i < NUM_CHARS; i++)
			_surfaces[i]->SetOpacity(opacity);
	 }

	 /// return the height of a charachter (it's the same for all the set, so it's also the height 
	 /// of a text line
	 Uint16 GetCharH() { return _surfaces[0]->GetH(); };
	 /// return the width of a text
	 Uint16 GetTextW(std::string txt);

	 /// update all the loaded surfaces
	 void UpdateTexs(bool del_old_tex = true)
	 {
		for(int i = 0; i < NUM_CHARS ; i++)
			_surfaces[i]->UpdateTex(del_old_tex);
	 };
};

}
#endif
