#ifndef CLASS_MOVABLE_SURFACE
#define CLASS_MOVABLE_SURFACE

#include "Surface.h"

namespace graphic
{

/// A surface usable for animations
class MovableSurface : public virtual Surface
{
	public:
	 /// empty constructor
	 MovableSurface() {};
	 /// destructor
	 virtual ~MovableSurface() { };

	 /// set INITIAL position of the upper left Point of the surface (x,y)
	 /// passing a Point
	 virtual void SetInitialPosition(Point position) { SetInitialPosition(position.x, position.y); };
	 /// set INITIAL position of the upper left Point of the surface (x,y)
	 /// passing x and y values
	 virtual void SetInitialPosition(Sint16 x, Sint16 y)
	 {
		// setting rectangle position
		_rectangle->x = x;
		_rectangle->y = y;
	 };
	 /// set the position of the upper left Point of the surface (x,y) passing a Point
	 void SetPosition(Point position)
	 {
		_rectangle->x = position.x;
		_rectangle->y = position.y;
	 }
	 /// set the position of the upper left Point of the surface (x,y) passing x and y values
	 void SetPosition(Sint16 x, Sint16 y)
	 {
		_rectangle->x = x;
		_rectangle->y = y;
	 }

	 /// move image of mov pixels along X and Y axis
	 void Move(Sint16 mov_x, Sint16 mov_y)
	 {
		// update current surface rectangle
		_rectangle->x += mov_x;
		_rectangle->y += mov_y;
	 }

	 /// move image of mov pixels along X axis
	 void MoveX(Sint16 mov) { _rectangle->x += mov; };
	 /// move image of mov pixels along Y axis
	 void MoveY(Sint16 mov) { _rectangle->y += mov; };

	 /// save background of Image before a blit
	 virtual void SaveBackground(Surface * src_surf = NULL) { };
	 /// restore a saved background of Image after a blit and update _bk_rectangle
	 virtual void RestoreBackground(Surface * dst_surf = NULL) { };

	 /// update altered rectangle of screen
	 virtual void UpdateScreen() { };
};

}
#endif
