#include "SDLBlitter.h"

// copy an entire surface on another one of the same size (NULL -> NULL)
void graphic::SDLBlitter::Copy(Surface * src, Surface * dest)
{
	SDLSurface * sdl_src = dynamic_cast<SDLSurface *> (src);

	if(dest == NULL)
	{
		SDLScreen * sdl_screen = static_cast<SDLScreen *> (screen);
		SDL_BlitSurface(sdl_src->_surface, NULL, sdl_screen->_surface, NULL);
	}
	else
	{
		SDLSurface * sdl_dest = dynamic_cast<SDLSurface *> (dest);
		SDL_BlitSurface(sdl_src->_surface, NULL, sdl_dest->_surface, NULL);
	}
}

// blit an entire surface on another one using surface position (NULL -> Rect)
void graphic::SDLBlitter::Blit(Surface * src, Surface * dest)
{
	SDLSurface * sdl_src = dynamic_cast<SDLSurface *> (src);
	SDLSurface * sdl_dest = NULL;
	SDLScreen * sdl_screen = NULL;

	if(dest == NULL)
		sdl_screen = static_cast<SDLScreen *> (screen);
	else
		sdl_dest = dynamic_cast<SDLSurface *> (dest);

	// backup rect
	SDL_Rect to;
	SDL_Rect * src_r = sdl_src->_rectangle;

	// copy src surface rect values into backup rect
	to.x = src_r->x;
	to.y = src_r->y;
	to.w = src_r->w;
	to.h = src_r->h;

	// blit on screen
	if(dest == NULL)
		SDL_BlitSurface(sdl_src->_surface, NULL, sdl_screen->_surface, &to);
	// blit on dest
	else
		SDL_BlitSurface(sdl_src->_surface, NULL, sdl_dest->_surface, &to);
}

// NULL -> Rect
// blit an entire surface on another one NOT using surface position, but dest_rect one
void graphic::SDLBlitter::Blit(Surface * src, SDL_Rect & dest_rect, Surface * dest)
{
	SDLSurface * sdl_src = dynamic_cast<SDLSurface *> (src);
	SDLSurface * sdl_dest = NULL;
	SDLScreen * sdl_screen = NULL;

	if(dest == NULL)
		sdl_screen = static_cast<SDLScreen *> (screen);
	else
		sdl_dest = dynamic_cast<SDLSurface *> (dest);

	// backup rect
	SDL_Rect to;

	// copy src surface rect values into backup rect
	to.x = dest_rect.x;
	to.y = dest_rect.y;
	to.w = dest_rect.w;
	to.h = dest_rect.h;

	// blit on screen
	if(dest == NULL)
		SDL_BlitSurface(sdl_src->_surface, NULL, sdl_screen->_surface, &to);
	// blit on dest
	else
		SDL_BlitSurface(sdl_src->_surface, NULL, sdl_dest->_surface, &to);
}

// blit part of a surface on another one using surface position for
// the destination (Rect -> rect)
void graphic::SDLBlitter::BlitPart(Surface * src, SDL_Rect & src_rect, Surface * dest)
{
	SDLSurface * sdl_src = dynamic_cast<SDLSurface *> (src);
	SDLSurface * sdl_dest = NULL;
	SDLScreen * sdl_screen = NULL;

	if(dest == NULL)
		sdl_screen = static_cast<SDLScreen *> (screen);
	else
		sdl_dest = dynamic_cast<SDLSurface *> (dest);

	// backup rect
	SDL_Rect from, to;
	SDL_Rect * src_r = sdl_src->_rectangle;

	// copy src surface rect values into backup rect
	from.x = src_rect.x;
	from.y = src_rect.y;
	from.w = src_rect.w;
	from.h = src_rect.h;

	to.x = src_r->x;
	to.y = src_r->y;
	to.w = src_rect.w;
	to.h = src_rect.h;

	// blit on screen
	if(dest == NULL)
		SDL_BlitSurface(sdl_src->_surface, &from, sdl_screen->_surface, &to);
	// blit on dest
	else
		SDL_BlitSurface(sdl_src->_surface, &from, sdl_dest->_surface, &to);
}

// blit part of a surface on part of another one NOT using surface position
// for the destination, but dest_rect.x and dest_rect.y (Rect -> Rect)
void graphic::SDLBlitter::BlitPart(Surface * src, SDL_Rect & src_rect,
							   SDL_Rect & dest_rect, Surface * dest)
{
	SDLSurface * sdl_src = dynamic_cast<SDLSurface *> (src);
	SDLSurface * sdl_dest = NULL;
	SDLScreen * sdl_screen = NULL;

	if(dest == NULL)
		sdl_screen = static_cast<SDLScreen *> (screen);
	else
		sdl_dest = dynamic_cast<SDLSurface *> (dest);

	// backup rect
	SDL_Rect from, to;

	// copy src surface rect values into backup rect
	from.x = src_rect.x;
	from.y = src_rect.y;
	from.w = src_rect.w;
	from.h = src_rect.h;

	// copy src surface rect values into backup rect
	to.x = dest_rect.x;
	to.y = dest_rect.y;
	to.w = dest_rect.w;
	to.h = dest_rect.h;

	// blit on screen
	if(dest == NULL)
		SDL_BlitSurface(sdl_src->_surface, &from, sdl_screen->_surface, &to);
	// blit on dest
	else
		SDL_BlitSurface(sdl_src->_surface, &from, sdl_dest->_surface, &to);
}

// blit the part of screen limited by src_rect on the dest surface
void graphic::SDLBlitter::SaveScreen(SDL_Rect & src_rect, Surface * dest)
{
	SDLScreen * sdl_screen = static_cast<SDLScreen *> (screen);
	SDLSurface * sdl_dest = dynamic_cast<SDLSurface *> (dest);

	// backup rect
	SDL_Rect from;

	// copy src surface rect values into backup rect
	from.x = src_rect.x;
	from.y = src_rect.y;
	from.w = src_rect.w;
	from.h = src_rect.h;

	// blit from screen to dest
	SDL_BlitSurface(sdl_screen->_surface, &from, sdl_dest->_surface, NULL);
}

// blit part of screen limited by src_rect on part of the dest surface
// limited by dest_rect
void graphic::SDLBlitter::SaveScreen(SDL_Rect & src_rect, SDL_Rect & dest_rect, Surface * dest)
{
	SDLScreen * sdl_screen = static_cast<SDLScreen *> (screen);
	SDLSurface * sdl_dest = dynamic_cast<SDLSurface *> (dest);

	// backup rect
	SDL_Rect from, to;

	// copy src surface rect values into backup rect
	from.x = src_rect.x;
	from.y = src_rect.y;
	from.w = src_rect.w;
	from.h = src_rect.h;

	// copy dest rect values into backup rect
	to.x = dest_rect.x;
	to.y = dest_rect.y;
	to.w = dest_rect.w;
	to.h = dest_rect.h;

	// blit from screen to dest
	SDL_BlitSurface(sdl_screen->_surface, &from, sdl_dest->_surface, &to);
}

