#ifndef CLASS_SDL_IMAGE
#define CLASS_SDL_IMAGE

#include <string>

#include <SDL_image.h>

#include "Image.h"
#include "SDLSurface.h"

namespace graphic
{
/// A surface loaded from an image file
class SDLImage : public virtual SDLSurface, public virtual Image
{
	private:
	 /// set visible limits of the image
	 void SetVisibleLimits();

	public:
	 /// base constructor, Create a new image from a file name, by default no alpha
	 SDLImage(std::string file);
	 /// create a new image from a file name setting a color as transparent
	 SDLImage(std::string file, SDL_Color trans_color);
	 /// destroyer
	 ~SDLImage() { };

	 /// return true if the pixel at (x,y) is transparent, false otherwise
	 bool IsTrans(Uint16 x, Uint16 y);

	 /// set the opacity of the surface
	 void SetOpacity(Uint8 opacity)
	 {
		 SDLSurface::SetOpacity(opacity);
		 SetVisibleLimits();
	 }
};
}
#endif
