#ifndef CLASS_SDL_MOVABLE_SURFACE
#define CLASS_SDL_MOVABLE_SURFACE

#include "Blitter.h"
#include "MovableSurface.h"
#include "SDLSurface.h"

namespace graphic
{

/// A surface usable for animations
class SDLMovableSurface : public virtual SDLSurface, public virtual MovableSurface
{
	protected:
	 /// backup surface, this surface is used to store the background of the movable
	 /// surface in order to restore it when needed
	 SDLSurface * _bk_surface;

	public:
	 /// empty constructor
	 SDLMovableSurface() {};
	 /// A new, not inizialized, surface
	 SDLMovableSurface(Uint16 w, Uint16 h, Uint32 flags = SDL_SWSURFACE,
					   bool trans = false, Uint16 bpp = screen->GetBitPP()) : SDLSurface(w, h, flags, trans, bpp)
	 { _bk_surface = new SDLSurface(w, h, flags, trans, bpp); };
	 /// Destructor
	 virtual ~SDLMovableSurface() { delete _bk_surface; };

	 /// Set INITIAL position of the upper left Point of the surface (x,y)
	 /// passing a Point
	 void SetInitialPosition(Point position) { SetInitialPosition(position.x, position.y); };
	 /// Set INITIAL position of the upper left Point of the surface (x,y)
	 /// passing x and y values
	 void SetInitialPosition(Sint16 x, Sint16 y)
	 {
		// setting rectangle position
		_rectangle->x = x;
		_rectangle->y = y;
	
		// setting backup rectangle position
		_bk_surface->SetPosition(x, y);
	 };

	 /// Save background of Image before a blit
	 void SaveBackground(Surface * src_surf = NULL);
	 /// Restore a saved background of Image after a blit and update _bk_rectangle
	 void RestoreBackground(Surface * dst_surf = NULL) { blitter->Blit(_bk_surface, dst_surf); };

	 /// Update altered rectangle of screen
	 void UpdateScreen();
};

}
#endif
