#include "SDLScreen.h"

// init screen surface
graphic::SDLScreen::SDLScreen(Uint16 w, Uint16 h, Uint32 flags)
{
	// saving flags value
	_flags = flags;

	// setting screen dimensions
	_width = w;
	_height = h;

	// setting screen mode, get a screen from SDL
	_surface = SDL_SetVideoMode(w, h, 0, _flags);

	// an error is occurred ==> throw an Exception object
	if(_surface == NULL)
		throw Exception("SDLScreen", SDL_GetError());
}

// update a given area of screen
void graphic::SDLScreen::UpdateRect(SDL_Rect & rect)
{
	if(rect.x < 0)
		rect.x = 0;
	if(rect.y < 0)
		rect.y = 0;
	if(rect.x + rect.w > _width)
		rect.x = _width - rect.w;
	if(rect.y + rect.h > _height)
		rect.y = _height - rect.h;

	SDL_UpdateRect(_surface, rect.x, rect.y, rect.w, rect.h);
}


// fill the surface with a color
void graphic::SDLScreen::Fill(SDL_Color & color)
{
	// map fill color
	Uint32 fill_color = SDL_MapRGB(_surface->format, color.r, color.g, color.b);
	// fill
	SDL_FillRect(_surface, NULL, fill_color);
}

// set window view mode
void graphic::SDLScreen::GoWindowed()
{
	// set window mode just if the screen is fullscreen
	if(_flags & SDL_FULLSCREEN)
	{
		_flags &= ~SDL_FULLSCREEN;

		_surface = SDL_SetVideoMode(_width, _height, 0, _flags);

		// an error is occurred
		if(_surface == NULL)
			throw Exception("Screen", SDL_GetError());
	}
}

// set fullscreen view mode
void graphic::SDLScreen::GoFullScreen()
{
	// set fullscreen just if the screen is in window mode
	if(!(_flags & SDL_FULLSCREEN))
	{
		_flags |= SDL_FULLSCREEN;

		_surface = SDL_SetVideoMode(_width, _height, 0, _flags);

		// an error is occurred
		if(_surface == NULL)
			throw Exception("Screen", SDL_GetError());
	}
}

// set screen resolution
void graphic::SDLScreen::SetResolution(Uint16 w, Uint16 h)
{
	// change resolution just if it really changes
	if(w != _width || h != _height)
	{
		_surface = SDL_SetVideoMode(w, h, 0, _flags);

		_width = w;
		_height = h;

		// an error is occurred
		if(_surface == NULL)
			throw Exception("Screen", SDL_GetError());
	}
}

// set screen flags
void graphic::SDLScreen::SetFlags(Uint32 flags)
{
	// saving flags value
	_flags = flags;

	// updating screen mode
	_surface = SDL_SetVideoMode(_width, _height, 0, flags);

	// an error is occurred
	if(_surface == NULL)
		throw Exception("Screen", SDL_GetError());
}
