#ifndef CLASS_SDL_SURFACE
#define CLASS_SDL_SURFACE

#include "Screen.h"
#include "Surface.h"

#define MAX_REST 5

namespace graphic
{

/// Basic SDL surface, just a rectangle that you can blit or fill with a color
class SDLSurface : public virtual Surface
{
	protected:
	 /// the surface
	 SDL_Surface * _surface;

	public:
	 /// base constructor, just set the surface opaque (visible)
	 SDLSurface() { _opacity =  SDL_ALPHA_OPAQUE; };
	 /// create a surface starting from a SDL_Surface
	 SDLSurface(SDL_Surface * surface);
	 /// a new, not inizialized, surface
	 SDLSurface(Uint16 w, Uint16 h, Uint32 flags = SDL_SWSURFACE, bool trans = false,
			 Uint16 bpp = screen->GetBitPP());
	 /// a new, not inizialized, surface
	 SDLSurface(Uint16 w, Uint16 h, SDL_PixelFormat * format, Uint32 flags = SDL_SWSURFACE);
	 /// surface data is freed
	 virtual ~SDLSurface() { SDL_FreeSurface(_surface); };

	 /// set the opacity of the surface
	 virtual void SetOpacity(Uint8 opacity);

	 /// fill the surface with a color
	 void Fill(SDL_Color & color)
	 {
		SDL_FillRect(_surface, NULL, SDL_MapRGB(_surface->format, color.r, color.g, color.b));
	 };
	 /// fill part of the surface with a color
	 void Fill(SDL_Color & color, SDL_Rect & rect);

	 /// get data from the pixel at (x,y) - NEED Lock/Unlock
	 Uint32 GetPixel(Sint16 x, Sint16 y);
	 /// set data of the pixel at (x,y) - NEED Lock/Unlock
	 void SetPixel(Sint16 x, Sint16 y, Uint32 data);

	 /// set R,G,B,A values of a pixel at (x,y) - NEED Lock/Unlock
	 void GetRGBA(Sint16 x, Sint16 y, Uint8 * r, Uint8 * g, Uint8 * b, Uint8 * a)
	 {
		 SDL_GetRGBA(GetPixel(x, y), _surface->format, r, g, b, a);
	 };
	 /// return Alpha value of a pixel at (x,y) - NEED Lock/Unlock
	 Uint8 GetAlpha(Sint16 x, Sint16 y);

	 /// lock the surface so to permit direct pixel access under SDL
	 void Lock()
	 {
		 if(SDL_MUSTLOCK(_surface))
			 SDL_LockSurface(_surface);
	 };
	 /// relase the previous lock
	 void Unlock()
	 {
		 if(SDL_MUSTLOCK(_surface))
			 SDL_UnlockSurface(_surface);
	 };

	 /// update texture
	 void UpdateTex(bool del_old_tex = true) { };

	 /// SDLBlitter is a friend of SDLSurface so to access to its data
	 friend class SDLBlitter;
};

}

#endif
