#ifndef CLASS_SCREEN
#define CLASS_SCREEN

#include <SDL.h>

#include "../Exception.h"

#define SDL_GRAPHIC 0
#define GL_GRAPHIC 1

namespace graphic
{

/// This is the screen (or window) where everything is blitted, it uses SDL
/// to get an available surface
class Screen
{
	protected:
	 /// screen width
	 int _width;
	 /// screen height
	 int _height;

	 /// screen surface
	 SDL_Surface * _surface;

	 /// screen flags
	 Uint32 _flags;

	public:
	 /// empty constructor
	 Screen() { };
	 /// destroyer -> video surface is freed by SDL_Quit
	 virtual ~Screen() { };

	 /// set screen flags
	 virtual void SetFlags(Uint32 flags) = 0;
	 /// get setted screen flags
	 Uint32 GetFlags() { return _flags; };

	 /// set screen resolution
	 virtual void SetResolution(Uint16 w, Uint16 h) = 0;

	 /// go in fullscreen mode
	 virtual void GoFullScreen() = 0;
	 /// go in windowed mode
	 virtual void GoWindowed() = 0;

	 /// get the Bits Per Pixel format (8 - 16 - 24 - 32)
	 Uint8  GetBitPP() { return _surface->format->BitsPerPixel; };

	 /// update the entire screen
	 virtual void Update() = 0;
	 /// update a given area of screen
	 virtual void UpdateRect(SDL_Rect & rect) = 0;

	 /// fill the surface with a color
	 virtual void Fill(SDL_Color & color) = 0;

	 /// clear the screen
	 virtual void Clear() = 0;

	 /// return a value that rapresents the current graphic mode
	 int GetGraphicMode()
	 {
		if(_flags & SDL_OPENGL)
			return GL_GRAPHIC;
		else
			return SDL_GRAPHIC;
	 };

	 /// return screen width
	 int GetW() { return _width; };
	 /// return screen height
	 int GetH() { return _height; };
};

/// external pointer to the screen
extern Screen * screen;

}
#endif
