#ifndef CLASS_TEXT
#define CLASS_TEXT

#include <sstream>
#include <string>

#include <SDL_ttf.h>

#include "../Exception.h"

#define NUM_CHARS 223
#define FIRST_CHAR 32
#define LAST_CHAR 255

namespace graphic
{

/// this class manage a text loading all the characters needed to write
class Text
{
	protected:
	 /// opacity value of the text, the value range is defined between
	 /// SDL_ALPHA_TRANSPARENT and SDL_ALPHA_OPAQUE
	 Uint8 _opacity;

	public:
	 /// empty constructor
	 Text() { _opacity = SDL_ALPHA_OPAQUE; };
	 /// destroyer
	 virtual ~Text() {};

	 /// write a text starting from (x, y) on a Surface
	 //virtual void WriteText(const std::string & text, Sint16 x, Sint16 y, Surface * dest = NULL) = 0;
	 virtual void WriteText(std::string text, Sint16 x, Sint16 y, Surface * dest = NULL) = 0;
	 /// write a text starting from start Point on a Surface
	 //void WriteText(const std::string & text, Point & start, Surface * dest = NULL)
	 void WriteText(std::string text, Point & start, Surface * dest = NULL)
	 {
		WriteText(text, start.x, start.y, dest);
	 };

	 /// write a number starting from (x, y) on a Surface
	 void WriteNumber(int num, Sint16 x, Sint16 y, Surface * dest = NULL)
	 {
		 std::ostringstream os1;
		 os1 << num;
		 WriteText(os1.str(), x, y, dest);
	 };
	 /// write a number starting from start Point on a Surface
	 void WriteNumber(int num, Point & start, Surface * dest = NULL)
	 {
		WriteNumber(num, start.x, start.y, dest);
	 };

	 /// return the height of a charachter (it's the same for all the chars, so it's also the height 
	 /// of a text line
	 virtual Uint16 GetCharH() = 0;

	 /// return the width of a text
	 virtual Uint16 GetTextW(std::string txt) = 0;
	 /// return the width of a number
	 Uint16 GetNumberW(int num)
	 {
		 std::ostringstream os1;
		 os1 << num;
		 return GetTextW(os1.str());
	 };

	 /// set the opacity of the text
	 virtual void SetOpacity(Uint8 opacity) = 0;
	 /// get the opacity value
	 Uint8 GetOpacity() { return _opacity; };

	 /// update all the loaded surfaces
	 virtual void UpdateTexs(bool del_old_tex = true) = 0;
};

}
#endif
