/* Mars, Land of No Mercy - http://www.marnomercy.org */

#include <bitset>
#include <string>
#include <vector>

#include "graphic/GLBlitter.h"
#include "graphic/GLImage.h"
#include "graphic/GLMovableImage.h"
#include "graphic/GLScreen.h"
#include "graphic/GLSurface.h"
#include "graphic/GLText.h"

#include "Building.h"
#include "DataTypes.h"
#include "Functions.h"
#include "IsoFow.h"
#include "IsoGrid.h"
#include "IsoMiniMap.h"
#include "IsoMouse.h"
#include "IsoObjectsMap.h"
#include "IsoTilesMap.h"
#include "MiniMapPanel.h"
#include "Path.h"
#include "Pathfind.h"
#include "PathHighlighter.h"
#include "PathNode.h"
#include "SceneRenderer.h"
#include "SelectionHighlighter.h"
#include "SelectionData.h"
#include "StatisticalValue.h"
#include "SceneElement.h"
#include "TileSet.h"
#include "Timer.h"
#include "Unit.h"

#include "Exception.h"

// minimum offset betwenn screen and widgets
#define SW_OFFSET 5

// map dimensions
#define ROWS 50
#define COLS 50

// transparency value of the mmap
#define MM_TRANS 100

using namespace std;
using namespace graphic;

Screen * graphic::screen;
Blitter * graphic::blitter;

int main(int argc, char **argv)
{
	// init SDL
	SDL_Init(SDL_INIT_VIDEO | SDL_INIT_TIMER);
	// init SDL_ttf
	TTF_Init();
	// when exit, execute SDL_Quit and TTF_Quit to restore everything
	atexit(TTF_Quit);
	atexit(SDL_Quit);

	// init/load data ==> possible exceptions
	try
	{
		// init screen
		screen = new GLScreen(RIS1_W, RIS1_H, SDL_SWSURFACE);
	}
	catch(Exception e)
	{
		e.PrintError();
		exit(-1);
	}

	// a new blitter
	blitter = new GLBlitter();

	// options (resolution & tileset)
	Options opt;

	// default res is 800x600
	opt.ris.w = RIS1_W;
	opt.ris.h = RIS1_H;

	// start menu
	menu(opt);

	// exit
	if(!(opt.choice))
		return 0;

	// change resolution
	if(opt.ris.w != screen->GetW())
		screen->SetResolution(opt.ris.w, opt.ris.h);

	SDL_Color magenta = {0xFF, 0, 0xFF, 0};
	SDL_Color yellow = {0xFF, 0xFF, 0, 0};
	SDL_Color orange = {0xFF, 0xCC, 0, 0};

	// select tileset to load
	string dir;
	if(opt.choice == 1)
		dir = "data/img/tiles/128x64/terrain_01/";
	else if(opt.choice == 2)
		dir = "data/img/tiles/128x64/terrain_02/";
	else if(opt.choice == 3)
		dir = "data/img/tiles/128x64/terrain_03/";

	// minimap tiles
	string dir_mm = "data/img/tiles/minimap/";

	// --- ELEMENTS ---
	// scrolling directions
	std::bitset<4> arrows;
	Image * img ,* img2, * img3;
	Text * txt, * txt2;
	StatisticalValue * stat;

	// pointers used by selection
	Element * temp_elem = NULL;
	FactionElement * sel_elem = NULL;
	Unit * sel_unit = NULL;
	Building * sel_build = NULL;

	bool elem_selected = false;

	CellInd * temp_cell = NULL;

	// SCENE ELEMENT 1
	vector < Image * > views_se1;
	try
	{
		img = new GLImage("data/img/scene_elements/mech_ruins_1/01.png");
		views_se1.push_back(img);
		img = new GLImage("data/img/scene_elements/mech_ruins_1/01.png");
		views_se1.push_back(img);
	}
	catch(Exception e)
	{
		e.PrintError();
		exit(-1);
	}

	vector < StatisticalValue * > stats_se1;
	stat = new StatisticalValue(300);
	stats_se1.push_back(stat);

	SceneElement element1(1, 1, views_se1, stats_se1);

	// SCENE ELEMENT 2
	vector < Image * > views_se2;
	try
	{
		img = new GLImage("data/img/scene_elements/fuel_drums_1/01.png");
		views_se2.push_back(img);
		img = new GLImage("data/img/scene_elements/fuel_drums_1/02.png");
		views_se2.push_back(img);
	}
	catch(Exception e)
	{
		e.PrintError();
		exit(-1);
	}

	vector < StatisticalValue * > stats_se2;
	stat = new StatisticalValue(200);
	stats_se2.push_back(stat);

	SceneElement element2(1, 1, views_se2, stats_se2);

	// SCENE ELEMENT 3
	vector < Image * > views_se3;
	try
	{
		img = new GLImage("data/img/buildings/NGG/bunker_outpost_destr_1/01.png");
		views_se3.push_back(img);
		img = new GLImage("data/img/buildings/NGG/bunker_outpost_destr_1/02.png");
		views_se3.push_back(img);
	}
	catch(Exception e)
	{
		e.PrintError();
		exit(-1);
	}

	vector < StatisticalValue * > stats_se3;
	stat = new StatisticalValue(2000);
	stats_se3.push_back(stat);

	SceneElement element3(3, 3, views_se3, stats_se3);

	// SCENE ELEMENT 4
	vector < StatisticalValue * > stats_se4;
	stat = new StatisticalValue(200);
	stats_se4.push_back(stat);

	SceneElement element4(1, 1, views_se2, stats_se4);

	// SCENE ELEMENT 5
	vector < StatisticalValue * > stats_se5;
	stat = new StatisticalValue(200);
	stats_se5.push_back(stat);

	SceneElement element5(1, 1, views_se2, stats_se5);

	// BUILDING 1
	vector < Image * > views_b1;
	try
	{
		img = new GLImage("data/img/buildings/NGG/bunker_outpost_1/01.png");
		views_b1.push_back(img);
		img = new GLImage("data/img/buildings/NGG/bunker_outpost_1/02.png");
		views_b1.push_back(img);
		views_b1.push_back(img);
	}
	catch(Exception e)
	{
		e.PrintError();
		exit(-1);
	}

	vector < StatisticalValue * > stats_b1;
	// ENERGY
	stat = new StatisticalValue(1000);
	stats_b1.push_back(stat);
	// VISIBILITY RANGE
	stat = new StatisticalValue(2);
	stats_b1.push_back(stat);

	Building build1(3, 3, views_b1, stats_b1);
	build1.SetFaction(F_NGG);

	// BUILDING 2
	vector < Image * > views_b2;
	try
	{
		img = new GLImage("data/img/buildings/SWA/fortress_1/01.png");
		views_b2.push_back(img);
		img = new GLImage("data/img/buildings/SWA/fortress_1/02.png");
		views_b2.push_back(img);
		views_b2.push_back(img);
	}
	catch(Exception e)
	{
		e.PrintError();
		exit(-1);
	}

	vector < StatisticalValue * > stats_b2;
	// ENERGY
	stat = new StatisticalValue(2000);
	stats_b2.push_back(stat);
	// VISIBILITY RANGE
	stat = new StatisticalValue(2);
	stats_b2.push_back(stat);

	Building build2(3, 3, views_b2, stats_b2);
	build2.SetFaction(F_SWA);

	// BUILDING 3
	vector < Image * > views_b3;
	try
	{
		img = new GLImage("data/img/buildings/SWA/cannons_1/01.png");
		views_b3.push_back(img);
		img = new GLImage("data/img/buildings/SWA/cannons_1/02.png");
		views_b3.push_back(img);
		views_b3.push_back(img);
	}
	catch(Exception e)
	{
		e.PrintError();
		exit(-1);
	}

	vector < StatisticalValue * > stats_b3;
	// ENERGY
	stat = new StatisticalValue(500);
	stats_b3.push_back(stat);
	// VISIBILITY RANGE
	stat = new StatisticalValue(2);
	stats_b3.push_back(stat);

	Building build3(2, 2, views_b3, stats_b3);
	build3.SetFaction(F_SWA);

	// UNIT 1 - HEAVY MECH NGG
	vector < Image * > views_u1;
	try
	{
		img = new GLImage("data/img/units/NGG/mech_heavy_1/01.png");
		views_u1.push_back(img);
		img = new GLImage("data/img/units/NGG/mech_heavy_1/02.png");
		views_u1.push_back(img);
		img = new GLImage("data/img/units/NGG/mech_heavy_1/03.png");
		views_u1.push_back(img);
		img = new GLImage("data/img/units/NGG/mech_heavy_1/04.png");
		views_u1.push_back(img);
		img = new GLImage("data/img/units/NGG/mech_heavy_1/05.png");
		views_u1.push_back(img);
		img = new GLImage("data/img/units/NGG/mech_heavy_1/06.png");
		views_u1.push_back(img);
		img = new GLImage("data/img/units/NGG/mech_heavy_1/07.png");
		views_u1.push_back(img);
		img = new GLImage("data/img/units/NGG/mech_heavy_1/08.png");
		views_u1.push_back(img);
	}
	catch(Exception e)
	{
		e.PrintError();
		exit(-1);
	}

	vector < StatisticalValue * > stats_u1;
	// ENERGY
	stat = new StatisticalValue(300);
	stats_u1.push_back(stat);
	// VISIBILITY RANGE
	stat = new StatisticalValue(5);
	stats_u1.push_back(stat);
	// ACTION POINTS
	stat = new StatisticalValue(60);
	stats_u1.push_back(stat);
	Unit unit1(1, 1, views_u1, stats_u1);
	unit1.SetFaction(F_NGG);
	unit1.SetDirection(NORTH);

	// UNIT 2  - MEDIUM MECH NGG
	vector < Image * > views_u2;
	try
	{
		img = new GLImage("data/img/units/NGG/mech_medium_1/01.png");
		views_u2.push_back(img);
		img = new GLImage("data/img/units/NGG/mech_medium_1/02.png");
		views_u2.push_back(img);
		img = new GLImage("data/img/units/NGG/mech_medium_1/03.png");
		views_u2.push_back(img);
		img = new GLImage("data/img/units/NGG/mech_medium_1/04.png");
		views_u2.push_back(img);
		img = new GLImage("data/img/units/NGG/mech_medium_1/05.png");
		views_u2.push_back(img);
		img = new GLImage("data/img/units/NGG/mech_medium_1/06.png");
		views_u2.push_back(img);
		img = new GLImage("data/img/units/NGG/mech_medium_1/07.png");
		views_u2.push_back(img);
		img = new GLImage("data/img/units/NGG/mech_medium_1/08.png");
		views_u2.push_back(img);
	}
	catch(Exception e)
	{
		e.PrintError();
		exit(-1);
	}

	vector < StatisticalValue * > stats_u2;
	// ENERGY
	stat = new StatisticalValue(200);
	stats_u2.push_back(stat);
	// VISIBILITY RANGE
	stat = new StatisticalValue(6);
	stats_u2.push_back(stat);
	// ACTION POINTS
	stat = new StatisticalValue(60);
	stats_u2.push_back(stat);

	Unit unit2(1, 1, views_u2, stats_u2);
	unit2.SetFaction(F_NGG);
	unit2.SetDirection(NORTH);

	// UNIT 3 - LIGHT MECH NGG
	vector < Image * > views_u3;
	try
	{
		img = new GLImage("data/img/units/NGG/mech_light_1/01.png");
		views_u3.push_back(img);
		img = new GLImage("data/img/units/NGG/mech_light_1/02.png");
		views_u3.push_back(img);
		img = new GLImage("data/img/units/NGG/mech_light_1/03.png");
		views_u3.push_back(img);
		img = new GLImage("data/img/units/NGG/mech_light_1/04.png");
		views_u3.push_back(img);
		img = new GLImage("data/img/units/NGG/mech_light_1/05.png");
		views_u3.push_back(img);
		img = new GLImage("data/img/units/NGG/mech_light_1/06.png");
		views_u3.push_back(img);
		img = new GLImage("data/img/units/NGG/mech_light_1/07.png");
		views_u3.push_back(img);
		img = new GLImage("data/img/units/NGG/mech_light_1/08.png");
		views_u3.push_back(img);
	}
	catch(Exception e)
	{
		e.PrintError();
		exit(-1);
	}

	vector < StatisticalValue * > stats_u3;
	// ENERGY
	stat = new StatisticalValue(100);
	stats_u3.push_back(stat);
	// VISIBILITY RANGE
	stat = new StatisticalValue(6);
	stats_u3.push_back(stat);
	// ACTION POINTS
	stat = new StatisticalValue(70);
	stats_u3.push_back(stat);

	Unit unit3(1, 1, views_u3, stats_u3);
	unit3.SetFaction(F_NGG);
	unit3.SetDirection(NORTH);

	// UNIT 4 - HEAVY MECH SWA
	vector < Image * > views_u4;
	try
	{
		img = new GLImage("data/img/units/SWA/mech_heavy_1/01.png");
		views_u4.push_back(img);
		img = new GLImage("data/img/units/SWA/mech_heavy_1/02.png");
		views_u4.push_back(img);
		img = new GLImage("data/img/units/SWA/mech_heavy_1/03.png");
		views_u4.push_back(img);
		img = new GLImage("data/img/units/SWA/mech_heavy_1/04.png");
		views_u4.push_back(img);
		img = new GLImage("data/img/units/SWA/mech_heavy_1/05.png");
		views_u4.push_back(img);
		img = new GLImage("data/img/units/SWA/mech_heavy_1/06.png");
		views_u4.push_back(img);
		img = new GLImage("data/img/units/SWA/mech_heavy_1/07.png");
		views_u4.push_back(img);
		img = new GLImage("data/img/units/SWA/mech_heavy_1/08.png");
		views_u4.push_back(img);
	}
	catch(Exception e)
	{
		e.PrintError();
		exit(-1);
	}

	vector < StatisticalValue * > stats_u4;
	// ENERGY
	stat = new StatisticalValue(300);
	stats_u4.push_back(stat);
	// VISIBILITY RANGE
	stat = new StatisticalValue(5);
	stats_u4.push_back(stat);
	// ACTION POINTS
	stat = new StatisticalValue(60);
	stats_u4.push_back(stat);

	Unit unit4(1, 1, views_u4, stats_u4);
	unit4.SetFaction(F_SWA);
	unit4.SetDirection(NORTH_EAST);

	// UNIT 5  - MEDIUM MECH SWA
	vector < Image * > views_u5;
	try
	{
		img = new GLImage("data/img/units/SWA/mech_medium_1/01.png");
		views_u5.push_back(img);
		img = new GLImage("data/img/units/SWA/mech_medium_1/02.png");
		views_u5.push_back(img);
		img = new GLImage("data/img/units/SWA/mech_medium_1/03.png");
		views_u5.push_back(img);
		img = new GLImage("data/img/units/SWA/mech_medium_1/04.png");
		views_u5.push_back(img);
		img = new GLImage("data/img/units/SWA/mech_medium_1/05.png");
		views_u5.push_back(img);
		img = new GLImage("data/img/units/SWA/mech_medium_1/06.png");
		views_u5.push_back(img);
		img = new GLImage("data/img/units/SWA/mech_medium_1/07.png");
		views_u5.push_back(img);
		img = new GLImage("data/img/units/SWA/mech_medium_1/08.png");
		views_u5.push_back(img);
	}
	catch(Exception e)
	{
		e.PrintError();
		exit(-1);
	}

	vector < StatisticalValue * > stats_u5;
	// ENERGY
	stat = new StatisticalValue(200);
	stats_u5.push_back(stat);
	// VISIBILITY RANGE
	stat = new StatisticalValue(6);
	stats_u5.push_back(stat);
	// ACTION POINTS
	stat = new StatisticalValue(60);
	stats_u5.push_back(stat);

	Unit unit5(1, 1, views_u5, stats_u5);
	unit5.SetFaction(F_SWA);
	unit5.SetDirection(EAST);

	// UNIT 6 - LIGHT MECH SWA
	vector < Image * > views_u6;
	try
	{
		img = new GLImage("data/img/units/SWA/mech_light_1/01.png");
		views_u6.push_back(img);
		img = new GLImage("data/img/units/SWA/mech_light_1/02.png");
		views_u6.push_back(img);
		img = new GLImage("data/img/units/SWA/mech_light_1/03.png");
		views_u6.push_back(img);
		img = new GLImage("data/img/units/SWA/mech_light_1/04.png");
		views_u6.push_back(img);
		img = new GLImage("data/img/units/SWA/mech_light_1/05.png");
		views_u6.push_back(img);
		img = new GLImage("data/img/units/SWA/mech_light_1/06.png");
		views_u6.push_back(img);
		img = new GLImage("data/img/units/SWA/mech_light_1/07.png");
		views_u6.push_back(img);
		img = new GLImage("data/img/units/SWA/mech_light_1/08.png");
		views_u6.push_back(img);
	}
	catch(Exception e)
	{
		e.PrintError();
		exit(-1);
	}

	vector < StatisticalValue * > stats_u6;
	// ENERGY
	stat = new StatisticalValue(100);
	stats_u6.push_back(stat);
	// VISIBILITY RANGE
	stat = new StatisticalValue(6);
	stats_u6.push_back(stat);
	// ACTION POINTS
	stat = new StatisticalValue(70);
	stats_u6.push_back(stat);

	Unit unit6(1, 1, views_u6, stats_u6);
	unit6.SetFaction(F_SWA);
	unit6.SetDirection(NORTH);

// --- END ELEMENTS ---

	// load the tileset of the map
	TileSet ts(dir, &magenta, false);

	// load the tileset of the minimap
	TileSet ts_mm(dir_mm, &magenta, false);

	// random generated tiles map
	IsoTilesMap tm(ROWS, COLS, &ts);

	// temporary objects map
	IsoObjectsMap om(&tm);

	// NGG
	om.AddObject(&element1, 10, 14);
	om.AddObject(&element2, 9, 4);
	om.AddObject(&element3, 20, 9);
	om.AddObject(&build1, 5, 4);
	om.AddObject(&unit1, 12, 7);
	om.AddObject(&unit2, 25, 25);
	om.AddObject(&unit3, 6, 20);

	// SWA
	om.AddObject(&element4, 38, 13);
	om.AddObject(&element5, 43, 10);
	om.AddObject(&build2, 40, 5);
	om.AddObject(&build3, 36, 20);
	om.AddObject(&unit4, 35, 12);
	om.AddObject(&unit5, 29, 19);
	om.AddObject(&unit6, 41, 16);

	build1.SubCurStat(STAT_ENERGY, 250);
	build2.SubCurStat(STAT_ENERGY, 1000);
	unit3.SubCurStat(STAT_ENERGY, 10);
	unit6.SubCurStat(STAT_ENERGY, 60);

	// building visibility
	om.ComputeObjVisibility(&build1, build1.GetCurStat(STAT_VISIBILITY));
	// unit visibility
	om.ComputeObjVisibility(&unit1, unit1.GetCurStat(STAT_VISIBILITY), unit1.GetDirection());
	// unit visibility
	om.ComputeObjVisibility(&unit2, unit2.GetCurStat(STAT_VISIBILITY), unit2.GetDirection());
	// unit visibility
	om.ComputeObjVisibility(&unit3, unit3.GetCurStat(STAT_VISIBILITY), unit3.GetDirection());

	// iso Fow
	try
	{
		img = new GLImage("data/img/tiles/128x64/black.png", magenta);
	}
	catch(Exception e)
	{
		e.PrintError();
		exit(-1);
	}
	IsoFow fow(&tm, img);

	// iso Grid
	try
	{
		img = new GLImage("data/img/grids/128x64/black_grid.png", magenta);
		img2 = new GLImage("data/img/grids/128x64/white_grid.png", magenta);
		img3 = new GLImage("data/img/grids/128x64/white_grid.png", magenta);
	}
	catch(Exception e)
	{
		e.PrintError();
		exit(-1);
	}
	IsoGrid grid(&tm, img, img2, img3);

	// iso Minimap
	IsoMiniMap mm(&tm, &om, &ts_mm);

	// -- SELECTION HIGHLIGHTER --
	vector< Image * > high_images;
	try
	{
		img = new GLImage("data/img/highlighter/01.png", magenta);
		high_images.push_back(img);
		img = new GLImage("data/img/highlighter/02.png", magenta);
		high_images.push_back(img);
		img = new GLImage("data/img/highlighter/03.png", magenta);
		high_images.push_back(img);
		img = new GLImage("data/img/highlighter/04.png", magenta);
		high_images.push_back(img);
	}
	catch(Exception e)
	{
		e.PrintError();
		exit(-1);
	}
	SelectionHighlighter sel(high_images);

	// -- SELECTION DATA --

	vector< Image * > sdata_images;
	try
	{
		img = new GLImage("data/img/sel_data/ebar_border.png", magenta);
		sdata_images.push_back(img);
		img = new GLImage("data/img/sel_data/ebar_good.png");
		sdata_images.push_back(img);
		img = new GLImage("data/img/sel_data/ebar_bad.png");
		sdata_images.push_back(img);
	}
	catch(Exception e)
	{
		e.PrintError();
		exit(-1);
	}
	SelectionData sel_data(sdata_images);

	// -- PATH HIGHLIGHTER
	try
	{
		img = new GLImage("data/img/tiles/128x64/green.png", magenta);
		img2 = new GLImage("data/img/tiles/128x64/red.png", magenta);
		img3 = new GLImage("data/img/tiles/128x64/yellow.png", magenta);
		txt = new GLText(RAP_FONT, yellow, RAP_FONT_SIZE);
		txt2 = new GLText(RAP_FONT, orange, RAP_FONT_SIZE);
	}
	catch(Exception e)
	{
		e.PrintError();
		exit(-1);
	}
	PathHighlighter path_h(&tm, img, img2, img3, txt, txt2);

	Pathfind path_finder(&tm);
	Path * gen_path = NULL;

	// current view (screen position on the map) rect
	SDL_Rect view = { ((tm.GetW() - screen->GetW()) / 2), 0, screen->GetW(), screen->GetH() };

	// upper limit of view
	Sint16 u_limit = -( view.h / 2);
	// bottom limit of view
	Sint16 b_limit = tm.GetH() - ( view.h / 2);
	// left limit of view
	Sint16 l_limit = -( view.w / 2);
	// right limit of view
	Sint16 r_limit = tm.GetW() - ( view.w / 2);

	//  -- WINDOW MINIMAP --
	// minimap window border
	MovableImage * mm_border_1024, * mm_border_800, * mm_border;
	// minimap view rect
	Image * mm_view_1024, * mm_view_800, * mm_view;

	SDL_Color black = { 0, 0, 0, 0};
	SDL_Rect rwindow = { 1, 1, 0, 0};

	try
	{
		mm_border_1024 = new GLMovableImage("data/img/mmap_border_1024.png");
		mm_border_800 = new GLMovableImage("data/img/mmap_border_800.png");
	
		mm_view_1024 = new GLImage("data/img/mmap_view_1024.png", magenta);
		mm_view_800 = new GLImage("data/img/mmap_view_800.png", magenta);
	}
	catch(Exception e)
	{
		e.PrintError();
		exit(-1);
	}

	// default data are for 800x600
	if(screen->GetW() == RIS1_W)
	{
		mm_border = mm_border_800;
		mm_view = mm_view_800;
	}
	else
	// resolution changed to 1024x768
	{
		mm_border = mm_border_1024;
		mm_view = mm_view_1024;
	}

	rwindow.w = mm_border->GetW() - 2;
	rwindow.h = mm_border->GetH() - 2;

	// set position of the minimap border, bottom-right corner
	mm_border->SetInitialPosition(screen->GetW() - mm_border->GetW(),
													screen->GetH() - mm_border->GetH());


	// visible limits of the minimap
	SDL_Rect mm_limit_1024 = { (mm.GetW() - (mm_border_1024->GetW() - 2)) / 2, 0,
	mm_border_1024->GetW() - 2, mm_border_1024->GetH() - 2 };
	SDL_Rect mm_limit_800 = { (mm.GetW() - (mm_border_800->GetW() - 2)) / 2, 0,
	mm_border_800->GetW() - 2, mm_border_800->GetH() - 2 };

	// set position of view rect into the minimap window 800x600
	if(screen->GetW() == RIS1_W)
		mm_view->SetPosition(view.x / mm.GetScale() - mm_limit_800.x, view.y / mm.GetScale() - mm_limit_800.y);
	// set position of view rect into the minimap window 1024x768
	else
		mm_view->SetPosition(view.x / mm.GetScale() - mm_limit_1024.x, view.y / mm.GetScale() - mm_limit_1024.y);

	// -- END WINDOW MINIMAP --

	// --MINIMAP PANEL  --
	MiniMapPanel * mm_panel_1024, * mm_panel_800, * mm_panel;
	mm_panel_1024 = new MiniMapPanel(&mm, mm_view_1024, mm_limit_1024);
	mm_panel_800 = new MiniMapPanel(&mm, mm_view_800, mm_limit_800);

	if(screen->GetW() == RIS1_W)
		mm_panel = mm_panel_800;
	else
		mm_panel = mm_panel_1024;

	mm_panel->SetRendererLimits(u_limit, b_limit, l_limit, r_limit);

	vector < MovableImage * > pointers;
	// isometric mouse
	MovableImage * mimg;
	try
	{
		mimg = new GLMovableImage("data/img/pointer.png");
		pointers.push_back(mimg);
		mimg = new GLMovableImage("data/img/pointer2.png");
		pointers.push_back(mimg);
	}
	catch(Exception e)
	{
		e.PrintError();
		exit(-1);
	}
	IsoMouse * mouse = new IsoMouse(pointers, &tm, screen->GetW() / 2, screen->GetH() / 2);
	mouse->SetView(view.x, view.y);

	// turn the system cursor off
	SDL_ShowCursor(0);

	// -- SCENE RENDERER INIT --
	SceneRenderer renderer(view, false);
	renderer.SetLimits(u_limit, b_limit, l_limit, r_limit);

	tm.UseBuffer();
	renderer.AddLayer(&tm);

	fow.UseBuffer();
	renderer.AddLayer(&fow);

	grid.UseBuffer();
	renderer.AddLayer(&grid);

	path_h.Active(false);
	renderer.AddLayer(&path_h);

	renderer.AddLayer(&om);

	sel.Active(false);
	renderer.AddLayer(&sel);

	sel_data.Active(false);
	renderer.AddLayer(&sel_data);

	renderer.RequestBlit();

	// -- END - SCENE RENDERER INIT --

	// init the buffer -> first blit
	renderer.RequestBufferRefresh();

	// -- WINDOW --
	mm_border->Fill(black, rwindow);
	mm_panel->Blit(rwindow, mm_border);
	mm_border->SetOpacity(MM_TRANS);

	//  ROTATION
	FloatPoint * f_origin;
	int orientation;

	// MOVEMENT
	PathNode * cur_pnode = NULL;
	bool checkp_reached = false;
	// unit movement destination
	Sint16 x_dest = 0;
	Sint16 y_dest = 0;
	int mov;

	bool done = false;
	bool active = true;
	
	bool unit_moving = false;

	// drag movement of the mouse
	int drag_mov_x, drag_mov_y;
	Point prev_mouse_pos = { 0, 0 };
	// flag that says if mouse has been clicked
	bool mouse_clicked = false;
	// flag that says if mouse is dragging
	bool dragging = false;
	// flag that says if the minimap window is opaque/ON or transparent/OFF
	bool mmap_on = false;

	SDL_Event event;

	// timer used for frame control
	Timer timer(DEFAULT_FPS);
	timer.Start();

	while(!done)
	{
		while(SDL_PollEvent(&event))
		{
			switch(event.type)
			{
				// exit from the window
				case SDL_QUIT:
					done = true;
					break;

				case SDL_ACTIVEEVENT:
					if (event.active.gain == 0)
						active = false;
					else if(event.active.state == SDL_APPMOUSEFOCUS)
						active = true;
					break;

				// press a key
				case SDL_KEYDOWN :
					switch(event.key.keysym.sym)
					{
						case SDLK_ESCAPE:
							done = true; 
							break;

						// change mouse pointer
						case SDLK_1:
							mouse->SetPointer(0);
							mouse->UpdateAndBlit();
							break;

						// change mouse pointer
						case SDLK_2:
							mouse->SetPointer(1);
							mouse->UpdateAndBlit();
							break;

						// go in fullscreen mode
						case SDLK_f:
							screen->GoFullScreen();

							// UPDATE TEXTURES
							renderer.UpdateLayersTexs();

							#ifdef WIN32
							mm_border_1024->UpdateTex(false);
							mm_border_800->UpdateTex(false);
							mm_view_1024->UpdateTex(false);
							mm_view_800->UpdateTex(false);

							for(unsigned int z = 0; z < pointers.size(); z++)
								pointers[z]->UpdateTex(false);

							for(unsigned int z = 0; z < views_se1.size(); z++)
								views_se1[z]->UpdateTex(false);
							for(unsigned int z = 0; z < views_se2.size(); z++)
								views_se2[z]->UpdateTex(false);
							for(unsigned int z = 0; z < views_se3.size(); z++)
								views_se3[z]->UpdateTex(false);
							for(unsigned int z = 0; z < views_b1.size(); z++)
								views_b1[z]->UpdateTex(false);
							for(unsigned int z = 0; z < views_b2.size(); z++)
								views_b2[z]->UpdateTex(false);
							for(unsigned int z = 0; z < views_b3.size(); z++)
								views_b3[z]->UpdateTex(false);
							for(unsigned int z = 0; z < views_u1.size(); z++)
								views_u1[z]->UpdateTex(false);
							for(unsigned int z = 0; z < views_u2.size(); z++)
								views_u2[z]->UpdateTex(false);
							for(unsigned int z = 0; z < views_u3.size(); z++)
								views_u3[z]->UpdateTex(false);
							for(unsigned int z = 0; z < views_u4.size(); z++)
								views_u4[z]->UpdateTex(false);
							for(unsigned int z = 0; z < views_u5.size(); z++)
								views_u5[z]->UpdateTex(false);
							for(unsigned int z = 0; z < views_u6.size(); z++)
								views_u6[z]->UpdateTex(false);
							#endif

							// request to blit the scene	
							renderer.RequestBufferRefresh();
							break;

						// switch grid 
						case SDLK_g:
							if(grid.IsActive())
								grid.Active(false);
							else
								grid.Active();

							// request to blit the scene	
							renderer.RequestBufferRefresh();
							break;

						// go in window mode
						case SDLK_w:
							screen->GoWindowed();

							// UPDATE TEXTURES
							renderer.UpdateLayersTexs();

							#ifdef WIN32
							mm_border_1024->UpdateTex(false);
							mm_border_800->UpdateTex(false);
							mm_view_1024->UpdateTex(false);
							mm_view_800->UpdateTex(false);

							for(unsigned int z = 0; z < pointers.size(); z++)
								pointers[z]->UpdateTex(false);

							for(unsigned int z = 0; z < views_se1.size(); z++)
								views_se1[z]->UpdateTex(false);
							for(unsigned int z = 0; z < views_se2.size(); z++)
								views_se2[z]->UpdateTex(false);
							for(unsigned int z = 0; z < views_se3.size(); z++)
								views_se3[z]->UpdateTex(false);
							for(unsigned int z = 0; z < views_b1.size(); z++)
								views_b1[z]->UpdateTex(false);
							for(unsigned int z = 0; z < views_b2.size(); z++)
								views_b2[z]->UpdateTex(false);
							for(unsigned int z = 0; z < views_b3.size(); z++)
								views_b3[z]->UpdateTex(false);
							for(unsigned int z = 0; z < views_u1.size(); z++)
								views_u1[z]->UpdateTex(false);
							for(unsigned int z = 0; z < views_u2.size(); z++)
								views_u2[z]->UpdateTex(false);
							for(unsigned int z = 0; z < views_u3.size(); z++)
								views_u3[z]->UpdateTex(false);
							for(unsigned int z = 0; z < views_u4.size(); z++)
								views_u4[z]->UpdateTex(false);
							for(unsigned int z = 0; z < views_u5.size(); z++)
								views_u5[z]->UpdateTex(false);
							for(unsigned int z = 0; z < views_u6.size(); z++)
								views_u6[z]->UpdateTex(false);
							#endif

							// request to blit the scene	
							renderer.RequestBufferRefresh();
							break;


						// 800x600
						case SDLK_F1:
							if(screen->GetW() == RIS1_W)
								break;

							// update screen dimension
							screen->SetResolution(RIS1_W, RIS1_H);

							// UPDATE TEXTURES
							renderer.UpdateLayersTexs();

							#ifdef WIN32
							mm_border_1024->UpdateTex(false);
							mm_border_800->UpdateTex(false);
							mm_view_1024->UpdateTex(false);
							mm_view_800->UpdateTex(false);

							for(unsigned int z = 0; z < pointers.size(); z++)
								pointers[z]->UpdateTex(false);

							for(unsigned int z = 0; z < views_se1.size(); z++)
								views_se1[z]->UpdateTex(false);
							for(unsigned int z = 0; z < views_se2.size(); z++)
								views_se2[z]->UpdateTex(false);
							for(unsigned int z = 0; z < views_se3.size(); z++)
								views_se3[z]->UpdateTex(false);
							for(unsigned int z = 0; z < views_b1.size(); z++)
								views_b1[z]->UpdateTex(false);
							for(unsigned int z = 0; z < views_b2.size(); z++)
								views_b2[z]->UpdateTex(false);
							for(unsigned int z = 0; z < views_b3.size(); z++)
								views_b3[z]->UpdateTex(false);
							for(unsigned int z = 0; z < views_u1.size(); z++)
								views_u1[z]->UpdateTex(false);
							for(unsigned int z = 0; z < views_u2.size(); z++)
								views_u2[z]->UpdateTex(false);
							for(unsigned int z = 0; z < views_u3.size(); z++)
								views_u3[z]->UpdateTex(false);
							for(unsigned int z = 0; z < views_u4.size(); z++)
								views_u4[z]->UpdateTex(false);
							for(unsigned int z = 0; z < views_u5.size(); z++)
								views_u5[z]->UpdateTex(false);
							for(unsigned int z = 0; z < views_u6.size(); z++)
								views_u6[z]->UpdateTex(false);
							#endif

							u_limit = -( screen->GetH() / 2);
							b_limit = tm.GetH() - ( screen->GetH() / 2);
							l_limit = -( screen->GetW() / 2);
							r_limit = tm.GetW() - ( screen->GetW() / 2);

							renderer.SetLimits(u_limit, b_limit, l_limit, r_limit);

							// update scene view rect
							renderer.UpdateViewDimensions(screen->GetW(), screen->GetH());

							// MINIMAP
							// update dimension-related data
							mm_border = mm_border_800;
							// set position of minimap window border
							mm_border->SetInitialPosition(screen->GetW() - mm_border->GetW(),
																			 screen->GetH() - mm_border->GetH());

							rwindow.w = mm_border->GetW() - 2;
							rwindow.h = mm_border->GetH() - 2;

							mm_panel = mm_panel_800;
							mm_panel->SetRendererLimits(u_limit, b_limit, l_limit, r_limit);
							mm_panel->UpdateView(renderer.GetViewX(), renderer.GetViewY());
							mm_border->Fill(black, rwindow);
							mm_panel->Blit(rwindow, mm_border);

							mmap_on = false;

							mm_border->SetOpacity(MM_TRANS);

							// refresh the scene
							renderer.RequestBufferRefresh();

							break;

						case SDLK_F2:
							if(screen->GetW() == RIS2_W)
								break;

							// update screen dimension
							screen->SetResolution(RIS2_W, RIS2_H);

							// UPDATE TEXTURES
							renderer.UpdateLayersTexs();

							#ifdef WIN32
							mm_border_1024->UpdateTex(false);
							mm_border_800->UpdateTex(false);
							mm_view_1024->UpdateTex(false);
							mm_view_800->UpdateTex(false);

							for(unsigned int z = 0; z < pointers.size(); z++)
								pointers[z]->UpdateTex(false);

							for(unsigned int z = 0; z < views_se1.size(); z++)
								views_se1[z]->UpdateTex(false);
							for(unsigned int z = 0; z < views_se2.size(); z++)
								views_se2[z]->UpdateTex(false);
							for(unsigned int z = 0; z < views_se3.size(); z++)
								views_se3[z]->UpdateTex(false);
							for(unsigned int z = 0; z < views_b1.size(); z++)
								views_b1[z]->UpdateTex(false);
							for(unsigned int z = 0; z < views_b2.size(); z++)
								views_b2[z]->UpdateTex(false);
							for(unsigned int z = 0; z < views_b3.size(); z++)
								views_b3[z]->UpdateTex(false);
							for(unsigned int z = 0; z < views_u1.size(); z++)
								views_u1[z]->UpdateTex(false);
							for(unsigned int z = 0; z < views_u2.size(); z++)
								views_u2[z]->UpdateTex(false);
							for(unsigned int z = 0; z < views_u3.size(); z++)
								views_u3[z]->UpdateTex(false);
							for(unsigned int z = 0; z < views_u4.size(); z++)
								views_u4[z]->UpdateTex(false);
							for(unsigned int z = 0; z < views_u5.size(); z++)
								views_u5[z]->UpdateTex(false);
							for(unsigned int z = 0; z < views_u6.size(); z++)
								views_u6[z]->UpdateTex(false);
							#endif

							u_limit = -( screen->GetH() / 2);
							b_limit = tm.GetH() - ( screen->GetH() / 2);
							l_limit = -( screen->GetW() / 2);
							r_limit = tm.GetW() - ( screen->GetW() / 2);

							renderer.SetLimits(u_limit, b_limit, l_limit, r_limit);

							// update scene view rect
							renderer.UpdateViewDimensions(screen->GetW(), screen->GetH());

							// MINIMAP
							// update dimension-related data
							mm_border = mm_border_1024;
							// set position of minimap window border
							mm_border->SetInitialPosition(screen->GetW() - mm_border->GetW(),
																			 screen->GetH() - mm_border->GetH());

							rwindow.w = mm_border->GetW() - 2;
							rwindow.h = mm_border->GetH() - 2;

							mm_panel = mm_panel_1024;
							mm_panel->SetRendererLimits(u_limit, b_limit, l_limit, r_limit);
							mm_panel->UpdateView(renderer.GetViewX(), renderer.GetViewY());
							mm_border->Fill(black, rwindow);
							mm_panel->Blit(rwindow, mm_border);

							mmap_on = false;

							mm_border->SetOpacity(MM_TRANS);

							// refresh the scene
							renderer.RequestBufferRefresh();

							break;

						case SDLK_DOWN:
							arrows.set(DOWN);
							break;

						case SDLK_UP:
							arrows.set(UP);
							break;

						case SDLK_LEFT:
							arrows.set(LEFT);
							break;

						case SDLK_RIGHT:
							arrows.set(RIGHT);
							break;

						// restore Unit Action Points
						case SDLK_SPACE:
							if(!unit_moving && sel_unit != NULL)
							{
								sel_unit->RestoreCurStat(STAT_ACTION);

								// recompute the path to update the remaining AP
								if(gen_path != NULL)
								{
									delete gen_path;

									gen_path = NULL;

									// not ask for path blit
									path_h.Active(false);

									// make a new path if it's possible
									if(mouse->IsInsideMap() && (tm.IsWalkable(mouse->GetRow(), mouse->GetCol()) || 
										tm.IsVisible(mouse->GetRow(), mouse->GetCol()) != VISIBLE) )
									{
										gen_path = path_finder.MakePath(sel_unit->GetRow(), sel_unit->GetCol(),
																								mouse->GetRow(), mouse->GetCol(),
																								sel_unit->GetDirection());

										path_h.SetPath(gen_path);

										path_h.SetRemainingAP(sel_unit->GetCurStat(STAT_ACTION) - gen_path->GetTotalCost());

										path_h.Active();
									}

									// refresh
									renderer.RequestBlit(BUFFERED_RENDERING);
								}
							}
							break;

						default:
							break;
					}
					break;

				// key released
				case SDL_KEYUP:
					switch(event.key.keysym.sym)
					{
						case SDLK_DOWN:
							arrows.set(DOWN, 0);
							break;

						case SDLK_UP:
							arrows.set(UP, 0);
							break;

						case SDLK_LEFT:
							arrows.set(LEFT, 0);
							break;

						case SDLK_RIGHT:
							arrows.set(RIGHT, 0);
							break;

						default:
							break;
					}
					break;

					case SDL_MOUSEMOTION:
						// store current mouse position
						mouse->SetPosition(event.button.x, event.button.y, renderer.GetViewX(), renderer.GetViewY());

						// cell changed, unit selected and not moving
						if(mouse->IsCellChanged() && !unit_moving && sel_unit && sel_unit->GetFaction() == F_NGG)
						{
							// the mouse is inside the map and the 
							// current cell is walkable or not visible
							if(mouse->IsInsideMap() && (tm.IsWalkable(mouse->GetRow(), mouse->GetCol()) || 
							tm.IsVisible(mouse->GetRow(), mouse->GetCol()) != VISIBLE) )
							{
								// a path has just been made
								if(gen_path != NULL)
									delete gen_path;

								// made a new path
								gen_path = path_finder.MakePath(sel_unit->GetRow(), sel_unit->GetCol(),
													mouse->GetRow(), mouse->GetCol(), sel_unit->GetDirection());

								// set the path highlighter and ask for blit
								path_h.SetPath(gen_path);
								path_h.SetRemainingAP(sel_unit->GetCurStat(STAT_ACTION) - gen_path->GetTotalCost());

								path_h.Active();

								// buffered render
								renderer.RequestBlit(BUFFERED_RENDERING);
							}
							// mouse is outside the map or on a not walkable cell => delete the current path
							else if(gen_path != NULL)
							{
								// delete path
								delete gen_path;
								gen_path = NULL;

								// not ask for path blit
								path_h.Active(false);

								// buffered render
								renderer.RequestBlit(BUFFERED_RENDERING);
							}
						}

						// mouse is clicked + motion => dragging
						if(mouse_clicked)
						{
							dragging = true;

							// -- IMPORTANT --
							// restore the screen under the mouse pointer so to remove the mouse
							// image before moving the window
							mouse->RestoreBackg();

							// restore background (screen) under the window
							mm_border->RestoreBackground();

							// compute the movement
							drag_mov_x = mouse->GetX() - prev_mouse_pos.x;
							drag_mov_y = mouse->GetY() - prev_mouse_pos.y;

							// check for boundary exception
							if((mm_border->GetX() - drag_mov_x) <= SW_OFFSET && drag_mov_x < 0)
								drag_mov_x = -mm_border->GetX();
							if((mm_border->GetX() + mm_border->GetW() + drag_mov_x) >= screen->GetW() - SW_OFFSET && 
							drag_mov_x > 0)
								drag_mov_x = screen->GetW() - (mm_border->GetX() + mm_border->GetW());
							if((mm_border->GetY() - drag_mov_y) <= SW_OFFSET && drag_mov_y < 0)
								drag_mov_y = -mm_border->GetY();
							if((mm_border->GetY() + mm_border->GetH() + drag_mov_y) >= screen->GetH() - SW_OFFSET && 
							drag_mov_y > 0)
								drag_mov_y = screen->GetH() - (mm_border->GetY() + mm_border->GetH());

							// move border and the window area
							mm_border->Move(drag_mov_x, drag_mov_y);

							// save new background (screen)
							mm_border->SaveBackground();
							// blit the window border
							blitter->Blit(mm_border);
							// update altered part of screen
							mm_border->UpdateScreen();

							// store previous mouse position
							prev_mouse_pos.x = mouse->GetX();
							prev_mouse_pos.y = mouse->GetY();

							// move the mouse and blit
							mouse->MoveAndBlit();
						}
						else
							// restore the screen and blit
							mouse->UpdateAndBlit();

						break;

					// mouse button clicked
					case SDL_MOUSEBUTTONDOWN:
						// LEFT click
						if(event.button.button == SDL_BUTTON_LEFT)
						{
							// maybe the first drag click
							prev_mouse_pos.x = mouse->GetX();
							prev_mouse_pos.y = mouse->GetY();

							if(mm_border->IsInside(mouse->GetX(), mouse->GetY()))
								mouse_clicked = true;
						}
						break;


					case SDL_MOUSEBUTTONUP:
						// LEFT click
						if(event.button.button == SDL_BUTTON_LEFT)
						{

							// click inside the minimap window
							if(mm_border->IsInside(mouse->GetX(), mouse->GetY()))
							{
								// and not dragging
								if(!dragging)
								{
									// minimap window is OFF
									if(!mmap_on)
									{
										mmap_on = true;
										mm_border->SetOpacity(SDL_ALPHA_OPAQUE);

										// update the scene
										renderer.RequestBlit(BUFFERED_RENDERING);
									}
									// minimap window is ON
									else
									{
										// manage the click on the minimap panel
										mm_panel->MouseClicked(mouse->GetX() - (mm_border->GetX() + rwindow.x),
																				mouse->GetY() - (mm_border->GetY() + rwindow.y));

										// update the render position according to minimap panel view
										renderer.UpdateViewPosition(mm_panel->GetViewX(), mm_panel->GetViewY());

										// update mouse view
										mouse->SetView(renderer.GetViewX(), renderer.GetViewY());

										renderer.RequestBufferRefresh();

										// refresh minimap window
										mm_border->Fill(black, rwindow);
										mm_panel->Blit(rwindow, mm_border);
									}
								}
							}
							// click outside the minimap
							else
							{
								// deactivate the minimap if it's ON
								if(mmap_on)
								{
									mmap_on = false;

									mm_border->SetOpacity(MM_TRANS);

									// update the scene
									renderer.RequestBlit(BUFFERED_RENDERING);
								}


								// delete previous selected cell
								if(temp_cell)
									delete temp_cell;

								//  object selected by the click
								temp_elem = (Element *)om.SelectObject(mouse->GetAbsX(), mouse->GetAbsY());
								// cell selected by the click
								temp_cell = mouse->GetCell();

								// select a cell outside the map or reclick on the selected elem ==> deselect object
								if((temp_cell == NULL || (sel_elem == temp_elem)) && elem_selected && !unit_moving)
								{
									elem_selected = false;

									sel_elem->Deselect();

									sel_elem = NULL;
									sel_unit = NULL;
									sel_build = NULL;

									// clear selection
									sel.ClearSelected();
									// clear selection
									sel_data.ClearSelected();

									// update scene
									sel.Active(false);
									sel_data.Active(false);
									renderer.RequestBlit(BUFFERED_RENDERING);

									if(gen_path != NULL)
									{
										delete gen_path;
										gen_path = NULL;
										path_h.Active(false);
									}
								}
								// select element
								else if(temp_elem != NULL && temp_elem->GetType() != SCENE_ELEMENT && !unit_moving)
								{
									elem_selected = true;

									if(sel_elem != NULL)
										sel_elem->Deselect();

									// store the pointer
									sel_elem = dynamic_cast <FactionElement *>(temp_elem);

									if(sel_elem->GetType() == UNIT)
									{
										sel_unit = dynamic_cast <Unit *>(temp_elem);
										sel_build = NULL;
									}
									else
									{
										sel_build = dynamic_cast <Building *>(temp_elem);
										sel_unit = NULL;
									}

									// set the selected flag of the objcet
									sel_elem->Select();

									// set the selected object in the selection layer
									sel.SetSelected(sel_elem);
									sel_data.SetSelected(sel_elem);

									// update scene
									sel.Active();
									sel_data.Active();
									renderer.RequestBlit(BUFFERED_RENDERING);

									if(gen_path != NULL)
									{
										delete gen_path;
										gen_path = NULL;
										path_h.Active(false);
									}
					 			}
								// start moving
								else if(gen_path!= NULL && !unit_moving &&
											sel_unit->GetCurStat(STAT_ACTION) >= gen_path->GetTotalCost() &&
											sel_unit->GetFaction() == F_NGG)
									unit_moving = true;
							}

							// mouse released
							mouse_clicked = false;
							// no dragging
							dragging = false;
							
						}

						// RIGHT click
						else if(event.button.button == SDL_BUTTON_RIGHT)
						{
							// rotate unit
							if(sel_unit != NULL && !unit_moving  && sel_unit->GetFaction() == F_NGG)
							{
								// relative origin => center of the unit image
								f_origin = iso2scr(sel_unit->GetX() + (sel_unit->GetImage()->GetW() / 2),
															sel_unit->GetY() + (sel_unit->GetImage()->GetH() / 2),
															tm.GetOriginX(),  tm.GetOriginY());

								// get orientation
								orientation = RelDirection((int)round(f_origin->x), (int)round(f_origin->y),
																		(int)round(mouse->GetAbsOrthoX()),
																		(int)round(mouse->GetAbsOrthoY()));

								delete f_origin;

								// rotate the unit
								if(sel_unit->GetType() == UNIT &&  orientation != SELF  &&
								orientation != sel_unit->GetDirection() &&
								sel_unit->GetCurStat(STAT_ACTION) >= ROTATION_COST)
								{
									sel_unit->SetDirection(orientation);

									sel_unit->SubCurStat(STAT_ACTION, ROTATION_COST);

									// reset map visibility
									tm.SetVisibility();
									// building
									om.ComputeObjVisibility(&build1, build1.GetCurStat(STAT_VISIBILITY));
									// unit
									om.ComputeObjVisibility(&unit1, unit1.GetCurStat(STAT_VISIBILITY), unit1.GetDirection());
									// unit
									om.ComputeObjVisibility(&unit2, unit2.GetCurStat(STAT_VISIBILITY), unit2.GetDirection());
									// unit visibility
									om.ComputeObjVisibility(&unit3, unit3.GetCurStat(STAT_VISIBILITY), unit3.GetDirection());

									// update the minimap
									mm.UpdateMap();
									mm_border->Fill(black, rwindow);
									mm_panel->Blit(rwindow, mm_border);

									// update the path
									if(mouse->IsInsideMap())
									{
										if(tm.IsWalkable(mouse->GetRow(), mouse->GetCol()) ||
										tm.IsVisible(mouse->GetRow(), mouse->GetCol()) != VISIBLE)
										{
											if(gen_path != NULL)
												delete gen_path;

											gen_path = path_finder.MakePath(sel_unit->GetRow(), sel_unit->GetCol(),
																									mouse->GetRow(), mouse->GetCol(), 
																									sel_unit->GetDirection());

											path_h.SetPath(gen_path);

											path_h.SetRemainingAP(sel_unit->GetCurStat(STAT_ACTION) - gen_path->GetTotalCost());
										}
									}

									renderer.RequestBufferRefresh();
								}
							}
						}
						break;

					default:
						break;
			}
		}

		if(active)
		{
			renderer.ManageScrolling(arrows, mouse->GetX(), mouse->GetY());

			// manage scrolling
			if(renderer.IsScrolling())
			{
				renderer.Scrolling(timer.GetElapsedTime());

				// update mouse view
				mouse->SetView(renderer.GetViewX(), renderer.GetViewY());

				// update minimap window
				mm_panel->UpdateView(renderer.GetViewX(), renderer.GetViewY());
				mm_border->Fill(black, rwindow);

				mm_panel->Blit(rwindow, mm_border);
			}

			// a unit is moving
			if(unit_moving)
			{
				// not blit selection highlighter
				sel.Active(false);
				sel_data.Active(false);

				// it needs to get the next path node
				if(cur_pnode == NULL)
				{
					cur_pnode = gen_path->PopNode();

					if(sel_unit->GetDirection() != cur_pnode->GetDirection())
						sel_unit->SetDirection(cur_pnode->GetDirection());

					if(gen_path->GetPathLen() == 0)
					{
						gen_path = NULL;
						path_h.Active(false);
					}

					// first node
					if(cur_pnode->GetRow() ==  sel_unit->GetRow()  && cur_pnode->GetCol() == sel_unit->GetCol())
					{
						if(sel_unit->GetDirection() != cur_pnode->GetDirection())
							sel_unit->SetDirection(cur_pnode->GetDirection());

						checkp_reached = true;
					}
					// the current node is not walkable
					else if(!tm.IsWalkable(cur_pnode->GetRow(), cur_pnode->GetCol()))
					{
						delete cur_pnode;
						cur_pnode = NULL;

						// stop moving
						unit_moving = false;

						delete gen_path;
						gen_path = NULL;
						path_h.Active(false);

						checkp_reached = true;

						// update the minimap
						mm.UpdateMap();
						mm_border->Fill(black, rwindow);
						mm_panel->Blit(rwindow, mm_border);

					}
					// move to the node
					else
					{
						sel_unit->SubCurStat(STAT_ACTION, cur_pnode->GetCost());

						checkp_reached = false;

						om.MoveObject(sel_unit, cur_pnode->GetRow(), cur_pnode->GetCol());

						x_dest = om.GetObjX(sel_unit);
						y_dest = om.GetObjY(sel_unit);

						// update the minimap
						mm.UpdateMap();
						mm_border->Fill(black, rwindow);
						mm_panel->Blit(rwindow, mm_border);

					}

					// reset map visibility
					tm.SetVisibility();
					// building
					om.ComputeObjVisibility(&build1, build1.GetCurStat(STAT_VISIBILITY));
					// unit
					om.ComputeObjVisibility(&unit1, unit1.GetCurStat(STAT_VISIBILITY), unit1.GetDirection());
					// unit
					om.ComputeObjVisibility(&unit2, unit2.GetCurStat(STAT_VISIBILITY), unit2.GetDirection());
					// unit visibility
					om.ComputeObjVisibility(&unit3, unit3.GetCurStat(STAT_VISIBILITY), unit3.GetDirection());

					renderer.RequestBufferRefresh();
				}

				// moving toward the next cell
				if(!checkp_reached)
				{
					mov = (int)ceil(MOVE_SPEED * timer.GetElapsedTime());
					checkp_reached = sel_unit->MoveTo(x_dest, y_dest, mov);
					renderer.RequestBlit(BUFFERED_RENDERING);
				}
				// cell reached
				else
				{
					delete cur_pnode;
					cur_pnode = NULL;

					checkp_reached = false;

					// it was the last cell of the path
					if(gen_path == NULL)
					{
						unit_moving = false;
						path_h.Active(false);

						// reblit the selection highlighter
						sel.Active();
						sel_data.Active();

						renderer.RequestBlit(BUFFERED_RENDERING);
					}
				}
			}

			// blit the scene
			if(renderer.NeedBlit())
			{
				renderer.BlitScene();

				// -- WINDOW --
				// save background (screen) of minimap window
				mm_border->SaveBackground();
				// blit minimap window border
				blitter->Blit(mm_border);

				// mouse pointer management
				mouse->Blit();

				screen->Update();
			}
		}

		// compute elapsed time
		timer.GoOn();
		// delay so to keep constant Frame Rate
		timer.Delay();
	}

	delete txt;
	delete txt2; 

	return 0;
}


